@@ -10,11 +10,14 @@
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
  * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.  This license includes without
- * limitation a license to do the foregoing actions under any patents of
- * the party supplying this software to the X Consortium.
+ * and/or sell copies of the Software subject to the restriction stated
+ * below, and to permit persons who receive copies from any such party to
+ * do so, with the only requirement being that this copyright notice remain
+ * intact.
+ * This license includes without limitation a license to do the foregoing
+ * actions under any patents of the party supplying this software to the 
+ * X Consortium.
+ *
  */
 
 #include "fig.h"
@@ -23,6 +26,7 @@
 #include "object.h"
 #include "paintop.h"
 #include "u_create.h"
+#include "u_draw.h"
 #include "u_search.h"
 #include "u_undo.h"
 #include "w_canvas.h"
@@ -148,7 +152,6 @@ add_splinearrow(spline, prev_point, selected_point)
     F_point	   *prev_point, *selected_point;
 {
     F_point	   *p;
-    F_control	   *c;
 
     if (prev_point == NULL) {	/* add backward arrow */
 	if (spline->back_arrow)
@@ -279,3 +282,8 @@ delete_splinearrow(spline, prev_point, selected_point)
     set_action_object(F_DELETE_ARROW_HEAD, O_SPLINE);
     set_modifiedflag();
 }
+
+
+
+
+
