@@ -10,11 +10,14 @@
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
  * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.  This license includes without
- * limitation a license to do the foregoing actions under any patents of
- * the party supplying this software to the X Consortium.
+ * and/or sell copies of the Software subject to the restriction stated
+ * below, and to permit persons who receive copies from any such party to
+ * do so, with the only requirement being that this copyright notice remain
+ * intact.
+ * This license includes without limitation a license to do the foregoing
+ * actions under any patents of the party supplying this software to the 
+ * X Consortium.
+ *
  */
 
 #include "fig.h"
@@ -246,10 +249,8 @@ delete_all()
     clean_up();
     set_action_object(F_DELETE, O_ALL_OBJECT);
 
-    /*
-     * Aggregate assignment between variables is allowed, but not from
-     * constant (weird!?)
-     */
+    /* in case the user is inside any compounds */
+    close_all_compounds();
 
     set_latestobjects(&objects);
 
