@@ -10,11 +10,14 @@
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
  * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.  This license includes without
- * limitation a license to do the foregoing actions under any patents of
- * the party supplying this software to the X Consortium.
+ * and/or sell copies of the Software subject to the restriction stated
+ * below, and to permit persons who receive copies from any such party to
+ * do so, with the only requirement being that this copyright notice remain
+ * intact.
+ * This license includes without limitation a license to do the foregoing
+ * actions under any patents of the party supplying this software to the 
+ * X Consortium.
+ *
  */
 
 #include "fig.h"
@@ -43,7 +46,7 @@ static		get_text(), sel_text();
 
 compound_selected()
 {
-    set_mousefun("tag object", "tag region", "compound tagged");
+    set_mousefun("tag object", "tag region", "compound tagged", "", "", "");
     canvas_kbd_proc = null_proc;
     canvas_locmove_proc = null_proc;
     init_searchproc_left(tag_object);
@@ -102,7 +105,7 @@ init_tag_region(x, y)
     int		    x, y;
 {
     init_box_drawing(x, y);
-    set_mousefun("", "final corner", "cancel");
+    set_mousefun("", "final corner", "cancel", "", "", "");
     draw_mousefun_canvas();
     canvas_leftbut_proc = null_proc;
     canvas_middlebut_proc = tag_region;
