@@ -10,11 +10,14 @@
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
  * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.  This license includes without
- * limitation a license to do the foregoing actions under any patents of
- * the party supplying this software to the X Consortium.
+ * and/or sell copies of the Software subject to the restriction stated
+ * below, and to permit persons who receive copies from any such party to
+ * do so, with the only requirement being that this copyright notice remain
+ * intact.
+ * This license includes without limitation a license to do the foregoing
+ * actions under any patents of the party supplying this software to the 
+ * X Consortium.
+ *
  */
 
 #include "fig.h"
@@ -79,6 +82,9 @@ scale_selected()
     reset_action_on();
 }
 
+char *BOX_SCL_MSG = "Can't use box scale on selected object";
+char *BOX_SCL2_MSG = "Can't use box scale on selected object; try putting it into a compound";
+
 static
 init_box_scale(obj, type, x, y, px, py)
     char	   *obj;
@@ -102,7 +108,7 @@ init_box_scale(obj, type, x, y, px, py)
 	    return;
 	break;
     default:
-	put_msg("Can't use box scale on selected object");
+	put_msg(BOX_SCL2_MSG);
 	return;
     }
     set_mousefun("new posn", "", "cancel", "", "", "");
@@ -176,7 +182,7 @@ init_boxscale_ellipse(x, y)
 
     if (cur_e->type == T_ELLIPSE_BY_RAD ||
 	cur_e->type == T_CIRCLE_BY_RAD) {
-	if (x == cur_e->start.x || y == cur_e->start.y) {
+	if (x == cur_e->start.x && y == cur_e->start.y) {
 	    put_msg("Center point selected, ignored");
 	    return False;
 	} else {
@@ -197,7 +203,7 @@ init_boxscale_ellipse(x, y)
     cur_angle = cur_e->angle;
 
     if (cur_x == fix_x || cur_y == fix_y) {
-	put_msg("Can't use box scale on selected object");
+	put_msg(BOX_SCL2_MSG);
 	return False;
     }
     set_action_on();
@@ -520,8 +526,6 @@ fix_scale_spline(x, y)
     old_s->next = cur_s;
     /* now change the original to become the new object */
     rescale_points(cur_s->points, x, y);
-    if (int_spline(cur_s))
-	remake_control_points(cur_s);
     /* redraw anything under the old spline */
     redisplay_spline(old_s);
     /* and the new one */
@@ -544,7 +548,7 @@ init_boxscale_compound(x, y)
     ymax = max2(cur_c->secorner.y, cur_c->nwcorner.y);
 
     if (xmin == xmax || ymin == ymax) {
-	put_msg("Can't use box scale on selected object");
+	put_msg(BOX_SCL_MSG);
 	return False;
     }
     set_action_on();
@@ -759,7 +763,7 @@ scale_line(l, sx, sy, refx, refy)
 	p->x = round(refx + (p->x - refx) * sx);
 	p->y = round(refy + (p->y - refy) * sy);
     }
-    /* now scale the radius */
+    /* now scale the radius for an arc-box */
     if (l->type == T_ARC_BOX) {
 	int h,w;
 	/* scale by the average of height/width factor */
@@ -782,18 +786,12 @@ scale_spline(s, sx, sy, refx, refy)
     int		    refx, refy;
 {
     F_point	   *p;
-    F_control	   *c;
 
     for (p = s->points; p != NULL; p = p->next) {
 	p->x = round(refx + (p->x - refx) * sx);
 	p->y = round(refy + (p->y - refy) * sy);
     }
-    for (c = s->controls; c != NULL; c = c->next) {
-	c->lx = refx + (c->lx - refx) * sx;
-	c->ly = refy + (c->ly - refy) * sy;
-	c->rx = refx + (c->rx - refx) * sx;
-	c->ry = refy + (c->ry - refy) * sy;
-    }
+
 }
 
 scale_arc(a, sx, sy, refx, refy)
@@ -890,8 +888,8 @@ init_boxscale_line(x, y)
 
     if (cur_l->type != T_BOX &&
 	cur_l->type != T_ARC_BOX &&
-	cur_l->type != T_PIC_BOX) {
-	put_msg("Can't use box scale on selected object");
+	cur_l->type != T_PICTURE) {
+	put_msg(BOX_SCL2_MSG);
 	return False;
     }
     p0 = cur_l->points;
@@ -903,7 +901,7 @@ init_boxscale_line(x, y)
     ymax = max3(p0->y, p1->y, p2->y);
 
     if (xmin == xmax || ymin == ymax) {
-	put_msg("Can't use box scale on selected object");
+	put_msg(BOX_SCL_MSG);
 	return False;
     }
     set_action_on();
@@ -993,7 +991,7 @@ fix_boxscale_line(x, y)
     new_l = copy_line(cur_l);
     draw_line(cur_l, ERASE);
     assign_newboxpoint(new_l, fix_x, fix_y, x, y);
-    if (new_l->type == T_PIC_BOX) {
+    if (new_l->type == T_PICTURE) {
 	if (signof(fix_x - from_x) != signof(fix_x - x))
 	    new_l->pic->flipped = 1 - new_l->pic->flipped;
 	if (signof(fix_y - from_y) != signof(fix_y - y))
@@ -1054,7 +1052,7 @@ init_scale_line()
     set_action_on();
     toggle_linemarker(cur_l);
     set_temp_cursor(crosshair_cursor);
-    if (cur_l->type == T_BOX || cur_l->type == T_PIC_BOX)
+    if (cur_l->type == T_BOX || cur_l->type == T_PICTURE)
 	boxsize_msg(2);	/* factor of 2 because measurement is from midpoint */
     elastic_scalepts(cur_l->points);
     canvas_locmove_proc = scaling_line;
