@@ -10,11 +10,14 @@
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
  * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.  This license includes without
- * limitation a license to do the foregoing actions under any patents of
- * the party supplying this software to the X Consortium.
+ * and/or sell copies of the Software subject to the restriction stated
+ * below, and to permit persons who receive copies from any such party to
+ * do so, with the only requirement being that this copyright notice remain
+ * intact.
+ * This license includes without limitation a license to do the foregoing
+ * actions under any patents of the party supplying this software to the 
+ * X Consortium.
+ *
  */
 
 #include "fig.h"
@@ -26,6 +29,8 @@
 #include "u_create.h"
 #include "version.h"
 #include "w_drawprim.h"
+#include "w_export.h"
+#include "w_print.h"
 #include "w_indpanel.h"
 #include "w_setup.h"
 #include "w_util.h"
@@ -39,10 +44,6 @@ char    *read_file_name;
 /* from w_msgpanel.c */
 extern Boolean	first_file_msg;
 
-/* from w_export.c and w_print.c */
-extern Widget	export_just_panel;
-extern Widget	print_just_panel;
-
 static char	Err_incomp[] = "Incomplete %s object at line %d.";
 
 static void        read_colordef();
@@ -127,6 +128,7 @@ read_fig(file_name, obj, merge, xoff, yoff)
     line_no = 0;
     read_file_name = file_name;
     first_file_msg = True;
+    uncompress_file(file_name);
     if ((fp = fopen(file_name, "r")) == NULL)
 	return (errno);
     else {
@@ -138,6 +140,66 @@ read_fig(file_name, obj, merge, xoff, yoff)
     }
 }
 
+/* unzip/uncompress file if necessary */
+
+uncompress_file(name)
+    char	   *name;
+{
+    char	   *compname = NULL;
+    char	    unc[PATH_MAX+20];	/* temp buffer for uncompress/gunzip command */
+    Boolean	    compr=False;
+    struct stat	    status;
+
+    /* see if the filename ends with .Z */
+    /* if so, generate uncompress command and use pipe (filetype = 1) */
+    if (strlen(name) > 2 && !strcmp(".Z", name + (strlen(name)-2))) {
+	sprintf(unc,"uncompress %s",name);
+	compr = True;
+    /* or with .z or .gz */
+    } else if ((strlen(name) > 3 && !strcmp(".gz", name + (strlen(name)-3))) ||
+	      ((strlen(name) > 2 && !strcmp(".z", name + (strlen(name)-2))))) {
+	sprintf(unc,"gunzip -q %s",name);
+	compr = True;
+    /* none of the above, see if the file with .Z or .gz or .z appended exists */
+    } else {
+	compname = (char*) malloc(strlen(name)+4);
+	strcpy(compname, name);
+	strcat(compname, ".Z");
+	if (!stat(compname, &status)) {
+	    sprintf(unc, "uncompress %s",compname);
+	    compr = True;
+	    strcpy(name,compname);
+	} else {
+	    strcpy(compname, name);
+	    strcat(compname, ".z");
+	    if (!stat(compname, &status)) {
+		sprintf(unc, "gunzip -q %s",compname);
+		compr = True;
+		strcpy(name,compname);
+	    } else {
+		strcpy(compname, name);
+		strcat(compname, ".gz");
+		if (!stat(compname, &status)) {
+		    sprintf(unc, "gunzip -q %s",compname);
+		    compr = True;
+		    strcpy(name,compname);
+		}
+	    }
+	}
+    }
+
+    /* do the uncompression/unzip if needed */
+    if (compr) {
+	char *c;
+	system(unc);
+	/* strip off the trailing .Z, .z or .gz */
+	c = strrchr(name,'.');
+	*c= '\0';
+	if (compname)
+	    free(compname);
+    }
+}
+
 readfp_fig(fp, obj, merge, xoff, yoff)
     FILE	   *fp;
     F_compound	   *obj;
@@ -155,7 +217,7 @@ readfp_fig(fp, obj, merge, xoff, yoff)
     for (i=0; i<MAX_USR_COLS; i++)
 	n_colorFree[i] = True;
 
-    bzero(obj, COMOBJ_SIZE);
+    bzero((char*)obj, COMOBJ_SIZE);
     if (fgets(buf, BUF_SIZE, fp) == 0)	/* version */
 	return -2;
     if (strncmp(buf, "#FIG", 4) == 0) { /* versions 1.4/later have #FIG in
@@ -184,7 +246,7 @@ readfp_fig(fp, obj, merge, xoff, yoff)
 
 	if (proto >= 30) {
 	    /* read Portrait/Landscape indicator now */
-	    if (fgets(buf, BUF_SIZE, fp) == 0) {
+	    if (read_line(fp) < 0) {
 		file_msg("No Portrait/Landscape specification");
 		return -1;		/* error */
 	    }
@@ -199,8 +261,8 @@ readfp_fig(fp, obj, merge, xoff, yoff)
 		}
 	    }
 
-	    /* read metric/inches OR Centering indicator now */
-	    if (fgets(buf, BUF_SIZE, fp) == 0) {
+	    /* read Centering indicator now */
+	    if (read_line(fp) < 0) {
 		file_msg("No Center/Flushleft specification");
 		return -1;		/* error */
 	    }
@@ -211,31 +273,107 @@ readfp_fig(fp, obj, merge, xoff, yoff)
 		    appres.flushleft = !strncasecmp(buf,"flush",5);
 		    /* and the printer and export justification labels */
 		    FirstArg(XtNlabel, just_items[appres.flushleft]);
-		    if (print_just_panel)
-			SetValues(print_just_panel);
-		    if (export_just_panel)
+		    if (export_popup)
 			SetValues(export_just_panel);
+		    if (print_popup)
+			SetValues(print_just_panel);
 		    /* NOW read metric/inches indicator */
-		    if (fgets(buf, BUF_SIZE, fp) == 0) {
+		    if (read_line(fp) < 0) {
 			file_msg("No Metric/Inches specification");
 			return -1;		/* error*/
 		    }
 		    line_no++;
 	    }
-	    /* set appres mode appropriately */
+	    /* set metric/inches mode appropriately */
+	    fig_units = (strncasecmp(buf,"metric",5) != 0);
 	    if (!merge) {
-		fig_units = (strncasecmp(buf,"metric",5) != 0);
+		/* set the units string for the length messages */
+		strcpy(cur_fig_units, fig_units ? "in" : "cm");
 		/* and set the rulers/grid accordingly */
 		if (fig_units != (int) appres.INCHES) {
 		    appres.INCHES = (Boolean) fig_units;
 		    reset_rulers();
 		    init_grid();
+		    setup_grid(cur_gridmode);
 		    /* change the label in the units widget */
 		    FirstArg(XtNlabel, appres.INCHES ? "in" : "cm");
 		    SetValues(unitbox_sw);
 		}
 	    }
+	    /* paper size, magnification, multiple page flag and transparent color
+	       (for GIF export) new in 3.2 */
+	    if (proto >= 32) {
+		/* read paper size now */
+		if (read_line(fp) < 0) {
+		    file_msg("No Paper size specification");
+		    return -1;		/* error */
+		}
+		/* parse the paper size */
+		if ((appres.papersize = parse_papersize(buf)) < 0) {
+		    file_msg("Illegal paper size in file, using default");
+		    appres.papersize = (appres.INCHES? PAPER_LETTER: PAPER_A4);
+		}
+		/* and the print and export paper size menus */
+		FirstArg(XtNlabel, paper_sizes[appres.papersize].fname);
+		if (export_popup)
+		    SetValues(export_papersize_panel);
+		if (print_popup)
+		    SetValues(print_papersize_panel);
+		line_no++;
+
+		/* read magnification now */
+		if (read_line(fp) < 0) {
+		    file_msg("No Magnification specification");
+		    return -1;		/* error */
+		}
+		appres.magnification = atoi(buf);
+		/* set the magnification in the export and print panels */
+		sprintf(buf,"%.2f",appres.magnification);
+		FirstArg(XtNstring, buf);
+		if (export_popup)
+		    SetValues(export_mag_text);
+		if (print_popup) {
+		    SetValues(print_mag_text);
+		    print_update_figure_size();
+		}
+		line_no++;
+
+		/* read multiple page flag now */
+		if (read_line(fp) < 0) {
+		    file_msg("No Multiple page flag specification");
+		    return -1;		/* error */
+		}
+		if (strncasecmp(buf,"multiple",8) != 0 &&
+		    strncasecmp(buf,"single",6) != 0) {
+		    file_msg("No Multiple page flag specification");
+		    return -1;
+		}
+		appres.multiple = (strncasecmp(buf,"multiple",8) == 0);
+		FirstArg(XtNlabel, multiple_pages[appres.multiple]);
+		if (export_popup)
+		    SetValues(export_multiple_panel);
+		if (print_popup)
+		    SetValues(print_multiple_panel);
+		line_no++;
+
+		/* read transparent color now */
+		if (read_line(fp) < 0) {
+		    file_msg("No Transparent color specification");
+		    return -1;		/* error */
+		}
+		appres.transparent = atoi(buf);
+#ifdef USE_GIF
+		/* make colorname from number */
+		set_color_name(appres.transparent, buf);
+		FirstArg(XtNlabel, buf);
+		if (export_popup)
+		    SetValues(export_transp_panel);
+#endif
+		line_no++;
+	    }
 	}
+
+	/* now read the figure itself */
 	status = read_objects(fp, obj);
 
     } else {
@@ -281,11 +419,20 @@ readfp_fig(fp, obj, merge, xoff, yoff)
        file_msg("Warning, because of a bug in version 3.0 you may need to offset");
        file_msg("your figure by 14 fig units in X and Y if this figure was");
        file_msg("converted from an older version of xfig.  See the File panel.");
-    }
-    if (proto == 30)
        scale_figure(obj,((float)PIX_PER_INCH)/obj->nwcorner.x,0);
-    else if (obj->nwcorner.x != PIX_PER_INCH)
-       scale_figure(obj,((float)PIX_PER_INCH)/obj->nwcorner.x,15);
+    } else if (obj->nwcorner.x != PIX_PER_INCH) {
+       if (proto == 21 && obj->nwcorner.x == 76 && !appres.INCHES)
+	  scale_figure(obj,((float)PIX_PER_INCH)/80,15); /* for 2.1.8S, HWS */
+       else
+          scale_figure(obj,((float)PIX_PER_INCH)/obj->nwcorner.x,15);
+    }
+
+    if (merge && (proto >= 30)) {		/* rescale for mixed units, HWS */
+       if (!fig_units && appres.INCHES)
+	   read_scale_compound(obj,((float)PIX_PER_INCH)/(2.54*PIX_PER_CM),0);
+       if (fig_units && !appres.INCHES)
+	   read_scale_compound(obj,(2.54*PIX_PER_CM)/((float)PIX_PER_INCH),0);
+    }
 
     /* shift the figure by the amount in the x and y offsets from the file panel */
     translate_compound(obj, xoff, yoff);
@@ -296,6 +443,29 @@ readfp_fig(fp, obj, merge, xoff, yoff)
     return (status);
 }
 
+parse_papersize(size)
+    char	   *size;
+{
+    int i,len;
+    char *c;
+
+    /* first get rid of trailing newline */
+    if (size[strlen(size)-1]=='\n')
+	size[strlen(size)-1]='\0';
+    /* then truncate at first space or parenthesis "(" in passed size */
+    if (((c=strchr(size,' '))!= NULL)||((c=strchr(size,'(')) != NULL)) {
+	*c ='\0';
+    }
+    len = strlen(size);
+    for (i=0; i<NUMPAPERSIZES; i++) {
+	if (strncasecmp(size,paper_sizes[i].sname,len) == 0)
+	    break;
+    }
+    if (i >= NUMPAPERSIZES)
+	return -1;
+    return i;
+}
+
 read_objects(fp, obj)
     FILE	   *fp;
     F_compound	   *obj;
@@ -660,6 +830,7 @@ read_lineobject(fp)
     int		    n, x, y, fa, ba, npts;
     int		    type, style, radius_flag;
     float	    thickness, wid, ht;
+    int		    ox, oy;
 
     if ((l = create_line()) == NULL)
 	return (NULL);
@@ -732,7 +903,7 @@ read_lineobject(fp)
 	l->back_arrow = new_arrow(type, style, thickness, wid, ht);
 	skip_comment(fp);
     }
-    if (l->type == T_PIC_BOX) {
+    if (l->type == T_PICTURE) {
 	line_no++;
 	if ((l->pic = create_pic()) == NULL) {
 	    free((char *) l);
@@ -744,6 +915,9 @@ read_lineobject(fp)
 	    return (NULL);
 	}
 	read_picobj(l->pic,l->pen_color);
+	/* we've read in a pic object - merge_file uses this info to decide
+	   whether or not to remap any picture colors in first figure */
+	pic_obj_read = True;
     } else
 	l->pic = NULL;
 
@@ -761,6 +935,8 @@ read_lineobject(fp)
 	free_linestorage(l);
 	return (NULL);
     }
+    ox = p->x;
+    oy = p->y;
     /* read subsequent points */
     if (proto < 22)
 	npts = 1000000;	/* loop until we find 9999 9999 for previous fig files */
@@ -773,6 +949,11 @@ read_lineobject(fp)
 	}
 	if (proto < 22 && x == 9999)
 	    break;
+	/* ignore identical consecutive points */
+	if (ox == x && oy == y)
+	    continue;
+	ox = x;
+	oy = y;
 	if ((q = create_point()) == NULL) {
 	    free_linestorage(l);
 	    return (NULL);
@@ -804,17 +985,17 @@ read_splineobject(fp)
 {
     F_spline	   *s;
     F_point	   *p, *q;
-    F_control	   *cp, *cq;
+    F_sfactor	   *cp, *cq;
     int		    c, n, x, y, fa, ba, npts;
     int		    type, style;
     float	    thickness, wid, ht;
-    float	    lx, ly, rx, ry;
+    double	    s_param,sbis;
 
     if ((s = create_spline()) == NULL)
 	return (NULL);
 
     s->points = NULL;
-    s->controls = NULL;
+    s->sfactors = NULL;
     s->for_arrow = s->back_arrow = NULL;
     s->next = NULL;
 
@@ -905,43 +1086,53 @@ read_splineobject(fp)
     p->next = NULL;
     skip_line(fp);
 
-    if (normal_spline(s))
-	return (s);
+    if (proto <= 31) {		/* to read files from version 3.1 and older */
+	if int_spline(s)
+	  for (;c>00;c-=2)    /* 2 control points per point given by user in
+			      version 3.1 and older : don't read them */
+	    skip_line(fp);
+	if (closed_spline(s)) {
+	    F_point *ptr   = s->points; 
+	    s->points = s->points->next;
+	    free (ptr);
+	}
+	if (! make_sfactors(s)) {
+	    free_splinestorage(s);
+	    return (NULL);
+	}
+	return(s);
+    }
 
     line_no++;
     skip_comment(fp);
-    /* Read controls */
-    if ((n = fscanf(fp, "%f%f%f%f", &lx, &ly, &rx, &ry)) != 4) {
+
+    /* Read sfactors - the s parameter for splines */
+    
+    if ((n = fscanf(fp, "%lf", &s_param)) != 1) {
 	file_msg(Err_incomp, "spline", line_no);
 	free_splinestorage(s);
 	return (NULL);
     };
-    if ((cp = create_cpoint()) == NULL) {
+    if ((cp = create_sfactor()) == NULL) {
 	free_splinestorage(s);
 	return (NULL);
     }
-    s->controls = cp;
-    cp->lx = lx;
-    cp->ly = ly;
-    cp->rx = rx;
-    cp->ry = ry;
+    s->sfactors = cp;
+    cp->s = s_param;
     while (--c) {
 	count_lines_correctly(fp);
-	if (fscanf(fp, "%f%f%f%f", &lx, &ly, &rx, &ry) != 4) {
+	if (fscanf(fp, "%lf", &s_param) != 1) {
 	    file_msg(Err_incomp, "spline", line_no);
 	    cp->next = NULL;
 	    free_splinestorage(s);
 	    return (NULL);
 	};
-	if ((cq = create_cpoint()) == NULL) {
+	if ((cq = create_sfactor()) == NULL) {
 	    cp->next = NULL;
 	    free_splinestorage(s);
 	    return (NULL);
 	}
-	cq->lx = lx;
-	cq->ly = ly;
-	cq->rx = rx;
-	cq->ry = ry;
+	cq->s=s_param;
 	cp->next = cq;
 	cp = cq;
     }
@@ -1229,7 +1420,7 @@ read_line(fp)
 skip_comment(fp)
     FILE	   *fp;
 {
-    char	    c;
+    int		    c;
 
     while ((c = fgetc(fp)) == '#')
 	skip_line(fp);
@@ -1583,3 +1774,29 @@ count_lines_correctly(fp)
     } while (cc==' '||cc=='\t');
     ungetc(cc,fp);
 }
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
