@@ -10,11 +10,14 @@
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
  * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.  This license includes without
- * limitation a license to do the foregoing actions under any patents of
- * the party supplying this software to the X Consortium.
+ * and/or sell copies of the Software subject to the restriction stated
+ * below, and to permit persons who receive copies from any such party to
+ * do so, with the only requirement being that this copyright notice remain
+ * intact.
+ * This license includes without limitation a license to do the foregoing
+ * actions under any patents of the party supplying this software to the 
+ * X Consortium.
+ *
  */
 
 #include "fig.h"
@@ -408,7 +411,7 @@ read_1_3_splineobject(fp)
     s->cap_style = CAP_BUTT;
     s->for_arrow = NULL;
     s->back_arrow = NULL;
-    s->controls = NULL;
+    s->sfactors = NULL;
     s->next = NULL;
     if ((p = create_point()) == NULL) {
 	free((char *) s);
@@ -425,9 +428,9 @@ read_1_3_splineobject(fp)
 	return (NULL);
     }
     if (t == DRAW_CLOSEDSPLINE)
-	s->type = T_CLOSED_NORMAL;
+	s->type = T_CLOSED_APPROX;
     else
-	s->type = T_OPEN_NORMAL;
+	s->type = T_OPEN_APPROX;
     if (f) {
 	s->for_arrow = forward_arrow();
 	s->for_arrow->wid = w;
@@ -456,6 +459,16 @@ read_1_3_splineobject(fp)
 	p->next = q;
 	p = q;
     }
+    if closed_spline(s) {	/* get rid of the first point (the last one
+				  has the same coordinates) */
+	F_point *ptr =s->points;
+	s->points=s->points->next;
+	free(ptr);
+    }
+    if (! make_sfactors(s)) {
+	free_splinestorage(s);
+	return (NULL);
+    }
     return (s);
 }
 
