@@ -10,21 +10,29 @@
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
  * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.  This license includes without
- * limitation a license to do the foregoing actions under any patents of
- * the party supplying this software to the X Consortium.
+ * and/or sell copies of the Software subject to the restriction stated
+ * below, and to permit persons who receive copies from any such party to
+ * do so, with the only requirement being that this copyright notice remain
+ * intact.
+ * This license includes without limitation a license to do the foregoing
+ * actions under any patents of the party supplying this software to the 
+ * X Consortium.
+ *
  */
 
 #include "fig.h"
 #include "resources.h"
 #include "mode.h"
 #include "object.h"
+#include "u_create.h"
 #include "w_setup.h"
+#include "w_zoom.h"
+
+
 
 extern int	num_object;
 
+
 write_file(file_name)
     char	   *file_name;
 {
@@ -40,12 +48,17 @@ write_file(file_name)
     num_object = 0;
     if (write_objects(fp)) {
 	put_msg("Error writing file %s, %s", file_name, sys_errlist[errno]);
+	exit (2);
 	return (-1);
     }
     put_msg("%d object(s) saved in \"%s\"", num_object, file_name);
     return (0);
 }
 
+
+/* for fig2dev */
+
+
 int
 write_objects(fp)
     FILE	   *fp;
@@ -94,6 +107,8 @@ write_objects(fp)
     }
     if (fclose(fp) == EOF)
 	return (-1);
+
+
     return (0);
 }
 
@@ -104,6 +119,10 @@ write_file_header(fp)
     fprintf(fp, appres.landscape? "Landscape\n": "Portrait\n");
     fprintf(fp, appres.flushleft? "Flush left\n": "Center\n");
     fprintf(fp, appres.INCHES? "Inches\n": "Metric\n");
+    fprintf(fp, "%s\n", paper_sizes[appres.papersize].sname);
+    fprintf(fp, "%.2f\n", appres.magnification);
+    fprintf(fp, "%s\n", appres.multiple? "Multiple": "Single");
+    fprintf(fp, "%d\n", appres.transparent);
     fprintf(fp, "%d %d\n", PIX_PER_INCH, 2);
     /* write the user color definitions (if any) */
     write_colordefs(fp);
@@ -183,7 +202,7 @@ write_ellipse(fp, e)
     if (e->radiuses.x == 0 || e->radiuses.y == 0)
 	return;
 
-    fprintf(fp, "%d %d %d %d %d %d %d %d %d %.7f %d %.3f %d %d %d %d %d %d %d %d\n",
+    fprintf(fp, "%d %d %d %d %d %d %d %d %d %.3f %d %.4f %d %d %d %d %d %d %d %d\n",
 	    O_ELLIPSE, e->type, e->style, e->thickness,
 	    e->pen_color, e->fill_color, e->depth, e->pen_style, e->fill_style,
 	    e->style_val, e->direction, e->angle,
@@ -218,8 +237,9 @@ write_line(fp, l)
     if (b)
 	fprintf(fp, "\t%d %d %.2f %.2f %.2f\n", b->type, b->style,
 		b->thickness, b->wid, b->ht);
-    if (l->type == T_PIC_BOX)
-	fprintf(fp, "\t%d %s\n", l->pic->flipped, l->pic->file);
+    if (l->type == T_PICTURE)
+	fprintf(fp, "\t%d %s\n", l->pic->flipped, 	/* prevent no filename */
+		((l->pic->file && *l->pic->file)? l->pic->file: "<empty>"));
 
     fprintf(fp, "\t");
     npts=0;
@@ -234,11 +254,13 @@ write_line(fp, l)
     fprintf(fp, "\n");
 }
 
+
+
 write_spline(fp, s)
     FILE	   *fp;
     F_spline	   *s;
 {
-    F_control	   *cp;
+    F_sfactor	   *cp;
     F_point	   *p;
     F_arrow	   *f, *b;
     int		   npts;
@@ -270,14 +292,18 @@ write_spline(fp, s)
     };
     fprintf(fp, "\n");
 
-    if (s->controls == NULL)
+    if (s->sfactors == NULL)
 	return;
+
+
+
+/* save new shape factor */
+
     fprintf(fp, "\t");
     npts=0;
-    for (cp = s->controls; cp != NULL; cp = cp->next) {
-	fprintf(fp, " %.2f %.2f %.2f %.2f",
-		cp->lx, cp->ly, cp->rx, cp->ry);
-	if (++npts >= 2 && cp->next != NULL) {
+    for (cp = s->sfactors; cp != NULL; cp = cp->next) {
+	fprintf(fp, " %.3f",cp->s);
+	if (++npts >= 8 && cp->next != NULL) {
 		fprintf(fp,"\n\t");
 		npts=0;
 	}
@@ -295,7 +321,7 @@ write_text(fp, t)
 
     if (t->length == 0)
 	return;
-    fprintf(fp, "%d %d %d %d %d %d %d %.7f %d %d %d %d %d ",
+    fprintf(fp, "%d %d %d %d %d %d %d %.4f %d %d %d %d %d ",
 	    O_TEXT, t->type, t->color, t->depth, t->pen_style,
 	    t->font, t->size, t->angle,
 	    t->flags, t->ascent+t->descent, t->length,
@@ -330,7 +356,12 @@ emergency_save(file_name)
     num_object = 0;
     if (write_objects(fp))
 	return (-1);
-    (void) fprintf(stderr, "xfig: %d object(s) saved in \"%s\"\n",
+    if (file_name[0] != '/') {
+	(void) fprintf(stderr, "xfig: %d object(s) saved in \"%s/%s\"\n",
+		   num_object, cur_dir, file_name);
+    } else {
+	(void) fprintf(stderr, "xfig: %d object(s) saved in \"%s\"\n",
 		   num_object, file_name);
+    }
     return (0);
 }
