@@ -8,11 +8,14 @@
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
  * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.  This license includes without
- * limitation a license to do the foregoing actions under any patents of
- * the party supplying this software to the X Consortium.
+ * and/or sell copies of the Software subject to the restriction stated
+ * below, and to permit persons who receive copies from any such party to
+ * do so, with the only requirement being that this copyright notice remain
+ * intact.
+ * This license includes without limitation a license to do the foregoing
+ * actions under any patents of the party supplying this software to the 
+ * X Consortium.
+ *
  */
 
 #include "fig.h"
@@ -20,6 +23,7 @@
 #include "object.h"
 #include "mode.h"
 #include "u_create.h"
+#include "w_color.h"
 #include "w_util.h"
 #include "f_neuclrtab.h"
 
@@ -42,8 +46,10 @@ emptyname_msg(name, msg)
 {
     int		    returnval;
 
-    if (returnval = emptyname(name))
+    if (returnval = emptyname(name)) {
 	put_msg("No file name specified, %s command ignored", msg);
+	beep();
+    }
     return (returnval);
 }
 
@@ -86,13 +92,14 @@ change_directory(path)
 	return (0);
     }
     if (chdir(path) == -1) {
-	put_msg("Can't go to directory %s, : %s", path, sys_errlist[errno]);
+	file_msg("Can't go to directory %s, : %s", path, sys_errlist[errno]);
 	return (1);
     }
     if (get_directory(cur_dir)) /* get cwd */
-	return (0);
+	return 0;
     else
-	return (1);
+	return 1;
+
 }
 
 get_directory(direct)
@@ -107,7 +114,7 @@ get_directory(direct)
 #endif
 
 #if defined(SYSV) || defined(SVR4)
-    if (getcwd(direct, 1024) == NULL) {	/* get current working dir */
+    if (getcwd(direct, PATH_MAX) == NULL) {	/* get current working dir */
 	put_msg("Can't get current directory");
 #else
     if (getwd(direct) == NULL) {/* get current working dir */
@@ -147,17 +154,21 @@ ok_to_write(file_name, op_name)
 		put_msg("Write permission for \"%s\" is denied", file_name);
 		return (0);
 	    } else {
-		if (warnexist) {
+	       if (warninput) {
+		 sprintf(string, "\"%s\" is an input file.\nDo you want to choose a new filename ?", file_name);
+		 popup_query(QUERY_YESCAN, string);
+	         return(0);
+		} else {
+		  if (warnexist) {
 		    sprintf(string, "\"%s\" already exists.\nDo you want to overwrite it?", file_name);
 		    if (!popup_query(QUERY_YESCAN, string)) {
 			put_msg("%s cancelled", op_name);
-			return (0);
+			return(0);
 		    }
-		}
-		/* !warnexist */
-		else {
+		   } else {
 			return(1);
-		}
+		   }
+		}  
 	    }
 	} else {
 	    put_msg("\"%s\" is read only", file_name);
@@ -185,7 +196,7 @@ basname(filename)
     }
 }
 
-/* remap the colors for all the EPS/XPM and GIFs in the compound passed */
+/* remap the colors for all the pictures in the compound passed */
 
 static	npixels;
 /* the colortable produced by the neural network code */
@@ -204,11 +215,11 @@ remap_image_two(obj, l)
     c->compounds = obj;
     c->lines = l;
     remap_imagecolors(c);
-    free(c);
+    free((char *) c);
 }
 
 static int	   scol, ncolors;
-static int	   num_oldcolors=-1;
+static int	   num_oldcolors = -1;
 static Boolean	   usenet;
 
 remap_imagecolors(obj)
@@ -233,14 +244,14 @@ remap_imagecolors(obj)
     if (ncolors == 0)
 	return;
 
-    put_msg("Remapping EPS/GIF/XPM colors...");
+    put_msg("Remapping picture colors...");
     app_flush();
     if (ncolors > appres.max_image_colors) {
-	ncolors = appres.max_image_colors;
-	usenet = True;
 	if (appres.DEBUG) 
 		fprintf(stderr,"More colors (%d) than allowed (%d), using neural net\n",
 				ncolors,appres.max_image_colors);
+	ncolors = appres.max_image_colors;
+	usenet = True;
     }
 
     /* if this is the first image, allocate the number of colorcells we need */
@@ -261,11 +272,11 @@ remap_imagecolors(obj)
 	}
 	num_oldcolors = avail_image_cols;
 	if (avail_image_cols < 2 && ncolors >= 2) {
-	    file_msg("Cannot allocate even 2 colors for EPS/GIF/XPM");
+	    file_msg("Cannot allocate even 2 colors for pictures");
 	    reset_cursor();
 	    num_oldcolors = -1;
 	    reset_cursor();
-	    put_msg("Remapping EPS/GIF/XPM colors...Done");
+	    put_msg("Remapping picture colors...Done");
 	    app_flush();
 	    return;
 	}
@@ -275,7 +286,7 @@ remap_imagecolors(obj)
 	int	stat;
 	int	mult = 1;
 
-	/* count total number of pixels in all the EPSs, GIFs and XPMs */
+	/* count total number of pixels in all the pictures */
 	count_pixels(obj);
 	/* initialize the neural network */
 	/* -1 means can't alloc memory, -2 or more means must have that many times
@@ -290,7 +301,7 @@ remap_imagecolors(obj)
 	    /* couldn't alloc memory for network */
 	    fprintf(stderr,"Can't alloc memory for neural network\n");
 	    reset_cursor();
-	    put_msg("Remapping EPS/GIF/XPM colors...Done");
+	    put_msg("Remapping picture colors...Done");
 	    app_flush();
 	    return;
 	}
@@ -298,7 +309,7 @@ remap_imagecolors(obj)
 	for (i=0; i<mult; i++)
 	    add_pixels(obj);
 
-	/* make a new colortable with the opimal colors */
+	/* make a new colortable with the optimal colors */
 	avail_image_cols = neu_clrtab(avail_image_cols);
 
 	/* now change the color cells with the new colors */
@@ -307,19 +318,27 @@ remap_imagecolors(obj)
 	    image_cells[i].green = (unsigned short) clrtab[i][N_GRN] << 8;
 	    image_cells[i].blue  = (unsigned short) clrtab[i][N_BLU] << 8;
 	}
-	XStoreColors(tool_d, tool_cm, image_cells, avail_image_cols);
+	YStoreColors(tool_cm, image_cells, avail_image_cols);
 	/* get the new, mapped indices for the image colormap */
 	remap_image_colormap(obj);
     } else {
-	/* simply store each image's colormap in the X colormap */ 
+	/*
+	 * Extract the RGB values from the image's colormap and allocate
+	 * the appropriate X colormap entries.
+	 */
 	scol = 0;	/* global color counter */
-	straight_cmap(obj);
-	XStoreColors(tool_d, tool_cm, image_cells, scol);
+	extract_cmap(obj);
+	for (i=0; i<scol; i++) {
+	    image_cells[i].flags = DoRed|DoGreen|DoBlue;
+	}
+	YStoreColors(tool_cm, image_cells, scol);
+	scol = 0;	/* global color counter */
+	readjust_cmap(obj);
 	if (appres.DEBUG) 
 	    fprintf(stderr,"Able to use %d colors without neural net\n",scol);
     }
     reset_cursor();
-    put_msg("Remapping EPS/GIF/XPM colors...Done");
+    put_msg("Remapping picture colors...Done");
     app_flush();
 }
 
@@ -332,14 +351,14 @@ count_colors(obj)
 	count_colors(c);
     }
     for (l = obj->lines; l != NULL; l = l->next) {
-	if (l->type == T_PIC_BOX && l->pic->bitmap != NULL &&
+	if (l->type == T_PICTURE && l->pic->bitmap != NULL &&
 	    l->pic->numcols > 0) {
 		ncolors += l->pic->numcols;
 	}
     }
 }
 
-/* allocate the color cells for the EPS/GIF/XPM images */
+/* allocate the color cells for the pictures */
 
 alloc_imagecolors(num)
     int		    num;
@@ -351,13 +370,8 @@ alloc_imagecolors(num)
     avail_image_cols = num;
     for (i=0; i<avail_image_cols; i++) {
 	image_cells[i].flags = DoRed|DoGreen|DoBlue;
-	if (!XAllocColorCells(tool_d, tool_cm, 
-		False, &plane_masks, 0, &image_cells[i].pixel, 1)) {
-	    if (!switch_colormap() ||
-		(!XAllocColorCells(tool_d, tool_cm,
-		   False, &plane_masks, 0, &image_cells[i].pixel, 1))) {
-			break;
-	    }
+	if (!alloc_color_cells(&image_cells[i].pixel, 1)) {
+	    break;
 	}
     }
     avail_image_cols = i;
@@ -372,13 +386,65 @@ count_pixels(obj)
 	count_pixels(c);
     }
     for (l = obj->lines; l != NULL; l = l->next) {
-	if (l->type == T_PIC_BOX && l->pic->bitmap != NULL &&
+	if (l->type == T_PICTURE && l->pic->bitmap != NULL &&
 	    l->pic->numcols > 0) {
 		npixels += l->pic->bit_size.x * l->pic->bit_size.y;
 	}
     }
 }
 
+readjust_cmap(obj)
+    F_compound	   *obj;
+{
+    F_line	   *l;
+    F_compound	   *c;
+    int		   i,j;
+
+    for (c = obj->compounds; c != NULL; c = c->next) {
+	readjust_cmap(c);
+    }
+    for (l = obj->lines; l != NULL; l = l->next) {
+	if (l->type == T_PICTURE && l->pic->bitmap != NULL &&
+	    l->pic->numcols > 0) {
+		for (i=0; i<l->pic->numcols; i++) {
+		    j = l->pic->cmap[i].pixel;
+		    l->pic->cmap[i].pixel = image_cells[j].pixel;
+		    scol++;
+		}
+		if (l->pic->pixmap)
+		    XFreePixmap(tool_d, l->pic->pixmap);
+		l->pic->pixmap = 0;		/* this will force regeneration of the pixmap */
+	}
+    }
+}
+
+extract_cmap(obj)
+    F_compound	   *obj;
+{
+    F_line	   *l;
+    F_compound	   *c;
+    int		   i;
+
+    for (c = obj->compounds; c != NULL; c = c->next) {
+	extract_cmap(c);
+    }
+    for (l = obj->lines; l != NULL; l = l->next) {
+	if (l->type == T_PICTURE && l->pic->bitmap != NULL &&
+	    l->pic->numcols > 0) {
+		for (i=0; i<l->pic->numcols; i++) {
+		    image_cells[scol].red   = l->pic->cmap[i].red << 8;
+		    image_cells[scol].green = l->pic->cmap[i].green << 8;
+		    image_cells[scol].blue  = l->pic->cmap[i].blue << 8;
+		    l->pic->cmap[i].pixel = scol;
+		    scol++;
+		}
+		if (l->pic->pixmap)
+		    XFreePixmap(tool_d, l->pic->pixmap);
+		l->pic->pixmap = 0;		/* this will force regeneration of the pixmap */
+	}
+    }
+}
+
 straight_cmap(obj)
     F_compound	   *obj;
 {
@@ -390,7 +456,7 @@ straight_cmap(obj)
 	straight_cmap(c);
     }
     for (l = obj->lines; l != NULL; l = l->next) {
-	if (l->type == T_PIC_BOX && l->pic->bitmap != NULL &&
+	if (l->type == T_PICTURE && l->pic->bitmap != NULL &&
 	    l->pic->numcols > 0) {
 		for (i=0; i<l->pic->numcols; i++) {
 		    image_cells[scol].red   = l->pic->cmap[i].red << 8;
@@ -419,7 +485,7 @@ add_pixels(obj)
 	add_pixels(c);
     }
     for (l = obj->lines; l != NULL; l = l->next) {
-	if (l->type == T_PIC_BOX && l->pic->bitmap != NULL &&
+	if (l->type == T_PICTURE && l->pic->bitmap != NULL &&
 	    l->pic->numcols > 0) {
 		/* now add each pixel to the sample list */
 		for (i=0; i<l->pic->bit_size.x * l->pic->bit_size.y; i++) {
@@ -446,7 +512,7 @@ remap_image_colormap(obj)
 	remap_image_colormap(c);
     }
     for (l = obj->lines; l != NULL; l = l->next) {
-	if (l->type == T_PIC_BOX && l->pic->bitmap != NULL &&
+	if (l->type == T_PICTURE && l->pic->bitmap != NULL &&
 	    l->pic->numcols > 0) {
 		for (i=0; i<l->pic->numcols; i++) {
 		    /* real color from the image */
@@ -465,8 +531,22 @@ remap_image_colormap(obj)
 }
 
 /* map the bytes in pic->bitmap to bits for monochrome display */
+/* DESTROYS original pic->bitmap */
 /* uses a Floyd-Steinberg algorithm from the pbmplus package */
 
+/* Here is the copyright notice:
+**
+** Copyright (C) 1989 by Jef Poskanzer.
+**
+** Permission to use, copy, modify, and distribute this software and its
+** documentation for any purpose and without fee is hereby granted, provided
+** that the above copyright notice appear in all copies and that both that
+** copyright notice and this permission notice appear in supporting
+** documentation.  This software is provided "as is" without express or
+** implied warranty.
+**
+*/
+
 #define FS_SCALE 1024
 #define HALF_FS_SCALE 512
 #define MAXVAL (256*256)
@@ -573,12 +653,82 @@ F_pic	*pic;
 	    nexterr = temperr;
 	    fs_direction = ! fs_direction;
 	}
-	free(pic->bitmap);
-	free(thiserr);
-	free(nexterr);
+	free((char *) pic->bitmap);
+	free((char *) thiserr);
+	free((char *) nexterr);
 	pic->bitmap = bptr;
 	/* monochrome */
 	pic->numcols = 0;
 		
 	return;
 }
+
+beep()
+{
+	XBell(tool_d,0);
+}
+
+#ifdef NOSTRSTR
+
+char *strstr(s1, s2)
+    char *s1, *s2;
+{
+    int len2;
+    char *stmp;
+
+    len2 = strlen(s2);
+    for (stmp = s1; *stmp != NULL; stmp++)
+	if (strncmp(stmp, s2, len2)==0)
+	    return stmp;
+    return NULL;
+}
+#endif
+
+/* strncasecmp and strcasecmp by Fred Appelman (Fred.Appelman@cv.ruu.nl) */
+
+#ifdef HAVE_NO_STRNCASECMP
+int
+strncasecmp(const char* s1, const char* s2, int n)
+{
+   char c1,c2;
+
+   while (--n>=0)
+   {
+	  /* Check for end of string, if either of the strings
+	   * is ended, we can terminate the test
+	   */
+	  if (*s1=='\0' && *s2!='\0') return -1; /* s1 ended premature */
+	  if (*s1!='\0' && *s2=='\0') return +1; /* s2 ended premature */
+
+	  c1=toupper(*s1++);
+	  c2=toupper(*s2++);
+	  if (c1<c2) return -1; /* s1 is "smaller" */
+	  if (c1>c2) return +1; /* s2 is "smaller" */
+   }
+   return 0;
+}
+
+#endif
+
+#ifdef HAVE_NO_STRCASECMP
+int
+strcasecmp(const char* s1, const char* s2)
+{
+   char c1,c2;
+
+   while (*s1 && *s2)
+   {
+	  c1=toupper(*s1++);
+	  c2=toupper(*s2++);
+	  if (c1<c2) return -1; /* s1 is "smaller" */
+	  if (c1>c2) return +1; /* s2 is "smaller" */
+   }
+   /* Check for end of string, if not both the strings ended they are 
+    * not the same. 
+	*/
+   if (*s1=='\0' && *s2!='\0') return -1; /* s1 ended premature */
+   if (*s1!='\0' && *s2=='\0') return +1; /* s2 ended premature */
+   return 0;
+}
+
+#endif
