@@ -1,3 +1,5 @@
+#ifndef FIG_H
+#define FIG_H
 /*
  * FIG : Facility for Interactive Generation of figures
  * Copyright (c) 1985 by Supoj Sutanthavibul
@@ -10,11 +12,14 @@
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
  * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.  This license includes without
- * limitation a license to do the foregoing actions under any patents of
- * the party supplying this software to the X Consortium.
+ * and/or sell copies of the Software subject to the restriction stated
+ * below, and to permit persons who receive copies from any such party to
+ * do so, with the only requirement being that this copyright notice remain
+ * intact.
+ * This license includes without limitation a license to do the foregoing
+ * actions under any patents of the party supplying this software to the 
+ * X Consortium.
+ *
  */
 
 /* For the X stuff, include only Xlib.h and Intrinsic.h here - 
@@ -42,11 +47,13 @@
 #include <ctype.h>
 #include <errno.h>
 
-#ifndef __bsdi__
+#if !defined(__bsdi__) && !defined(__NetBSD__)
 extern int	errno;
 extern int	sys_nerr;
+#if (! (defined(BSD) && (BSD >= 199306)))
 extern char    *sys_errlist[];
 #endif
+#endif
 
 extern char    *mktemp();
 
@@ -57,6 +64,37 @@ extern char    *mktemp();
 
 #include <X11/Xos.h>
 
+/* for those who have an older (R4) Xos.h, we need to include unistd.h here */
+
+/*
+ * Get open(2) constants
+ */
+#ifdef X_NOT_POSIX
+#include <fcntl.h>
+#ifdef USL
+#include <unistd.h>
+#endif /* USL */
+#ifdef CRAY
+#include <unistd.h>
+#endif /* CRAY */
+#ifdef MOTOROLA
+#include <unistd.h>
+#endif /* MOTOROLA */
+#ifdef SYSV386
+#include <unistd.h>
+#endif /* SYSV386 */
+#include <sys/file.h>
+#else /* X_NOT_POSIX */
+#if !defined(_POSIX_SOURCE) && defined(macII)
+#define _POSIX_SOURCE
+#include <fcntl.h>
+#undef _POSIX_SOURCE
+#else
+#include <fcntl.h>
+#endif
+#include <unistd.h>
+#endif /* X_NOT_POSIX else */
+
 #if XtSpecificationRelease > 4
 #include <X11/Xfuncs.h>
 #else
@@ -217,23 +255,9 @@ int bcmp();
 
 #endif /* XtSpecificationRelease > 4 */
 
-#ifndef X_NOT_STDC_ENV
-#include <string.h>
-#ifndef index
-#define index strchr
-#endif
-#ifndef rindex
-#define rindex strrchr
-#endif
-#else  /* X_NOT_STDC_ENV IS defined */
+#ifdef X_NOT_STDC_ENV
 #ifdef SYSV
 #include <string.h>
-#ifndef index
-#define index strchr
-#endif
-#ifndef rindex
-#define rindex strrchr
-#endif
 #else  /* NOT SYSV */
 #include <strings.h>
 #ifndef strchr
@@ -288,7 +312,7 @@ extern char *getenv();
 #ifdef _POSIX_SOURCE
 #include <limits.h>
 #else
-#if !defined(sun) || defined(sparc)
+#if !defined(sun) || defined(sparc) || (defined(SVR4) && defined(i386))
 #define _POSIX_SOURCE
 #include <limits.h>
 #undef _POSIX_SOURCE
@@ -323,6 +347,14 @@ extern char *getenv();
 
 #define		DEF_NAME	"unnamed.fig"
 
+#ifdef USE_INLINE
+#define INLINE inline
+#else
+#define INLINE
+#endif /* USE_INLINE */
+
+#endif /* FIG_H */
+
 #ifdef NOSTRSTR
 extern char *strstr();
 #endif
