@@ -1,3 +1,5 @@
+#ifndef FIGX_H
+#define FIGX_H
 /*
  * FIG : Facility for Interactive Generation of figures
  * Copyright (c) 1994 by Brian V. Smith
@@ -8,11 +10,14 @@
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
  * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.  This license includes without
- * limitation a license to do the foregoing actions under any patents of
- * the party supplying this software to the X Consortium.
+ * and/or sell copies of the Software subject to the restriction stated
+ * below, and to permit persons who receive copies from any such party to
+ * do so, with the only requirement being that this copyright notice remain
+ * intact.
+ * This license includes without limitation a license to do the foregoing
+ * actions under any patents of the party supplying this software to the 
+ * X Consortium.
+ *
  */
 
 #include <X11/cursorfont.h>
@@ -33,7 +38,9 @@
 #include <X11/Xaw/Scrollbar.h>
 #include <X11/Xaw/Sme.h>
 #include <X11/Xaw/SmeBSB.h>
+#include <X11/Xaw/SmeLine.h>
 #include <X11/Xaw/Toggle.h>
 #include <X11/Xaw/Paned.h>
 #include <X11/Xaw/Viewport.h>
 #include <X11/Xaw/List.h>
+#endif /* FIGX_H */
