@@ -10,11 +10,14 @@
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
  * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.  This license includes without
- * limitation a license to do the foregoing actions under any patents of
- * the party supplying this software to the X Consortium.
+ * and/or sell copies of the Software subject to the restriction stated
+ * below, and to permit persons who receive copies from any such party to
+ * do so, with the only requirement being that this copyright notice remain
+ * intact.
+ * This license includes without limitation a license to do the foregoing
+ * actions under any patents of the party supplying this software to the 
+ * X Consortium.
+ *
  */
 
 #include "fig.h"
@@ -33,8 +36,8 @@ int		latexline_mode = 0;
 int		latexarrow_mode = 0;
 int		autoforwardarrow_mode = 0;
 int		autobackwardarrow_mode = 0;
-int		cur_gridmode = GRID_0;
-int		cur_pointposn = P_MAGNET;
+int		cur_gridmode;
+int		cur_pointposn;
 int		posn_rnd[P_GRID3 + 1];
 int		posn_hlf[P_GRID3 + 1];
 int		grid_fine[GRID_3 + 1];
@@ -53,6 +56,7 @@ int		anypointposn = 0;
 int		figure_modified = 0;
 char		cur_fig_units[32];
 Boolean		warnexist = False;
+Boolean		warninput = False;
 
 /**********************	 global mode variables	************************/
 
@@ -68,38 +72,48 @@ Boolean		batch_exists = False;
 char		batch_file[32];
 
 char	       *lang_items[] = {
-    "box",     "latex",  "epic", "eepic", "eepicemu",
-    "pictex",  "ibmgl",  "eps",  "ps",    "pstex", 
-    "pstex_t", "textyl", "tpic", "pic",   "mf",
-    "gif",     "jpg",    "xbm",
+	"box",  "latex", "epic", "eepic", "eepicemu", "pictex",
+	"hpl",  "eps",   "ps",   "pstex", "textyl",   "tpic",
+	"pic",  "mf",    "acad", "pcx",   "png",      "gif",
+#ifdef USE_JPEG
+	"jpeg",
+#endif /* USE_JPEG */
+	"tiff",  "xbm",
 #ifdef USE_XPM
-    "xpm",
-#endif
-	};
+	"xpm",
+#endif /* USE_XPM */
+	"ppm"
+    };
 
 char	       *lang_texts[] = {
-    "LaTeX box (figure boundary)    ",
-    "LaTeX picture                  ",
-    "LaTeX picture + epic macros    ",
-    "LaTeX picture + eepic macros   ",
-    "LaTeX picture + eepicemu macros",
-    "PiCTeX macros                  ",
-    "IBMGL (or HPGL)                ",
-    "Encapsulated Postscript        ",
-    "Postscript                     ",
-    "Combined PS/LaTeX (PS part)    ",
-    "Combined PS/LaTeX (LaTeX part) ",
-    "Textyl \\special commands       ",
-    "TPIC                           ",
-    "PIC                            ",
-    "MF (MetaFont)                  ",
-    "GIF                            ",
-    "JPEG                           ",
-    "X11 Bitmap                     ",
+	"LaTeX box (figure boundary)    ",
+	"LaTeX picture                  ",
+	"LaTeX picture + epic macros    ",
+	"LaTeX picture + eepic macros   ",
+	"LaTeX picture + eepicemu macros",
+	"PiCTeX macros                  ",
+	"IBMGL (or HPGL)                ",
+	"Encapsulated Postscript        ",
+	"Postscript                     ",
+	"Combined PS/LaTeX (both parts) ",
+	"Textyl \\special commands       ",
+	"TPIC                           ",
+	"PIC                            ",
+	"MF   (MetaFont)                ",
+	"ACAD (AutoCad Slide)           ",
+	"PCX  (PC Paintbrush)           ",
+	"PNG                            ",
+	"GIF                            ",
+#ifdef USE_JPEG
+	"JPEG                           ",
+#endif /* USE_JPEG */
+	"TIFF (no compression)          ",
+	"X11 Bitmap (XBM)               ",
 #ifdef USE_XPM
-    "X11 Pixmap (XPM)               ",
-#endif
-	};
+	"X11 Pixmap (XPM)               ",
+#endif /* USE_XPM */
+	"Portable Pixmap (PPM)          ",
+    };
 
 /***************************  Mode Settings  ****************************/
 
@@ -126,7 +140,7 @@ int		cur_capstyle	= CAP_BUTT;
 float		cur_dashlength	= DEF_DASHLENGTH;
 float		cur_dotgap	= DEF_DOTGAP;
 float		cur_styleval	= 0.0;
-Color		cur_pencolor	= DEFAULT;
+Color		cur_pencolor	= BLACK;
 Color		cur_fillcolor	= WHITE;
 int		cur_boxradius	= DEF_BOXRADIUS;
 int		cur_fillstyle	= UNFILLED;
@@ -142,8 +156,8 @@ float		cur_elltextangle = 0.0;	/* text/ellipse input angle */
 /***************************  File Settings  ****************************/
 
 char		cur_dir[1024];
-char		cur_filename[200] = "";
-char		save_filename[200] = "";	/* to undo load */
+char		cur_filename[PATH_MAX] = "";
+char		save_filename[PATH_MAX] = "";	/* to undo load */
 char		cut_buf_name[100];
 char		file_header[32] = "#FIG ";
 
