@@ -1,3 +1,5 @@
+#ifndef OBJECT_H
+#define OBJECT_H
 /*
  * FIG : Facility for Interactive Generation of figures
  * Copyright (c) 1985 by Supoj Sutanthavibul
@@ -10,21 +12,34 @@
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
  * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.  This license includes without
- * limitation a license to do the foregoing actions under any patents of
- * the party supplying this software to the X Consortium.
+ * and/or sell copies of the Software subject to the restriction stated
+ * below, and to permit persons who receive copies from any such party to
+ * do so, with the only requirement being that this copyright notice remain
+ * intact.
+ * This license includes without limitation a license to do the foregoing
+ * actions under any patents of the party supplying this software to the 
+ * X Consortium.
+ *
  */
 
+/* values to signify color used for transparent GIF color */
+
+#define		TRANSP_BACKGROUND	-3	/* use background of figure as transp color */
+#define		TRANSP_NONE		-2	/* no transp color */
+
 /* DEFAULT is used for many things - font, color etc */
 
 #define		DEFAULT		      (-1)
+
 #define		SOLID_LINE		0
 #define		DASH_LINE		1
 #define		DOTTED_LINE		2
-#define		RUBBER_LINE		3
-#define		PANEL_LINE		4
+#define		DASH_DOT_LINE		3
+#define		DASH_2_DOTS_LINE	4
+#define		DASH_3_DOTS_LINE	5
+
+#define		RUBBER_LINE		11
+#define		PANEL_LINE		12
 
 #define		BLACK			0
 #define		BLUE			1
@@ -135,23 +150,34 @@ typedef struct f_arc {
 typedef struct f_pic {
     char	    file[PATH_MAX];
     int		    subtype;
+
 /* be sure to update NUM_PIC_TYPES if you add types */
-#ifdef USE_XPM
-#define NUM_PIC_TYPES	5
-#define T_PIC_PIXMAP	5
-#else
-#define NUM_PIC_TYPES	4
+/* These values are used internally, so changing them doesn't
+   affect any Fig files */
+
+enum {
+	T_PIC_NONE,
+	T_PIC_EPS,
+	T_PIC_GIF,
+#ifdef USE_JPEG
+	T_PIC_JPEG,
 #endif
-#define T_PIC_EPS	1
-#define T_PIC_GIF	2
-#define T_PIC_JPEG	3
-#define T_PIC_BITMAP	4
+	T_PIC_PCX,
+	T_PIC_XBM,
+#ifdef USE_XPM
+	T_PIC_XPM,
+#endif
+	LAST_PIC
+     } _pic_types;
+
+#define NUM_PIC_TYPES LAST_PIC-1
 
 #define PicSuccess	1
 #define FileInvalid    -2
+
     int		    flipped;
     unsigned char   *bitmap;
-    struct Cmap	    cmap[MAXCOLORMAPSIZE];  /* for GIF/XPM/JPEG files */
+    struct Cmap	    cmap[MAX_COLORMAP_SIZE];  /* for GIF/XPM/JPEG files */
     int		    numcols;		/* number of colors in cmap */
     float	    hw_ratio;
     int		    size_x, size_y;	/* picture size (fig units) */
@@ -177,7 +203,7 @@ typedef struct f_line {
 #define					T_BOX		2
 #define					T_POLYGON	3
 #define					T_ARC_BOX	4
-#define					T_PIC_BOX	5
+#define					T_PICTURE	5
     int		    style;
     int		    thickness;
     Color	    pen_color;
@@ -258,27 +284,34 @@ typedef struct f_text {
 
 #define		using_ps	(cur_textflags & PSFONT_TEXT)
 
-typedef struct f_control {
-    float	    lx, ly, rx, ry;
-    struct f_control *next;
+typedef struct f_shape {
+    double s;
+    struct f_shape *next;
 }
-		F_control;
+		F_sfactor;
 
 /* SEE NOTE AT TOP BEFORE CHANGING ANYTHING IN THE f_spline STRUCTURE */
 
 #define		int_spline(s)		(s->type & 0x2)
-#define		normal_spline(s)	(!(s->type & 0x2))
+#define         x_spline(s)	        (s->type & 0x4)
+#define		approx_spline(s)	(!(int_spline(s)|x_spline(s)))
 #define		closed_spline(s)	(s->type & 0x1)
 #define		open_spline(s)		(!(s->type & 0x1))
 
+#define S_SPLINE_ANGULAR 0.0
+#define S_SPLINE_APPROX 1.0
+#define S_SPLINE_INTERP (-1.0)
+
 typedef struct f_spline {
     int		    tagged;
     int		    distrib;
     int		    type;
-#define					T_OPEN_NORMAL	0
-#define					T_CLOSED_NORMAL 1
+#define					T_OPEN_APPROX	0
+#define					T_CLOSED_APPROX 1
 #define					T_OPEN_INTERP	2
 #define					T_CLOSED_INTERP 3
+#define                                 T_OPEN_XSPLINE  4
+#define                                 T_CLOSED_XSPLINE  5
     int		    style;
     int		    thickness;
     Color	    pen_color;
@@ -294,12 +327,10 @@ typedef struct f_spline {
 /* THE PRECEDING VARS MUST BE IN THE SAME ORDER IN f_arc, f_line and f_spline */
 
     /*
-     * For T_OPEN_NORMAL and T_CLOSED_NORMAL points are control points while
-     * they are knots for T_OPEN_INTERP and T_CLOSED_INTERP whose control
-     * points are stored in controls.
+     * "points" are control points. Shape factors are stored in "sfactors".
      */
     struct f_point *points;	/* this must immediately follow cap_style */
-    struct f_control *controls;
+    struct f_shape *sfactors;
     struct f_spline *next;
 }
 		F_spline;
@@ -314,6 +345,8 @@ typedef struct f_compound {
     struct f_spline *splines;
     struct f_text  *texts;
     struct f_arc   *arcs;
+    struct f_compound *parent;	/* for "enter/leave compound" */
+    struct f_compound *GABPtr;	/* Where original compound came from */
     struct f_compound *compounds;
     struct f_compound *next;
 }
@@ -334,7 +367,7 @@ typedef struct f_linkinfo {
 
 #define		ARROW_SIZE	sizeof(struct f_arrow)
 #define		POINT_SIZE	sizeof(struct f_point)
-#define		CONTROL_SIZE	sizeof(struct f_control)
+#define		CONTROL_SIZE	sizeof(struct f_shape)
 #define		ELLOBJ_SIZE	sizeof(struct f_ellipse)
 #define		ARCOBJ_SIZE	sizeof(struct f_arc)
 #define		LINOBJ_SIZE	sizeof(struct f_line)
@@ -358,26 +391,29 @@ typedef struct f_linkinfo {
 
 /********************* object masks for update  ************************/
 
-#define M_NONE			0x000
-#define M_POLYLINE_POLYGON	0x001
-#define M_POLYLINE_LINE		0x002
-#define M_POLYLINE_BOX		0x004	/* includes ARCBOX */
-#define M_SPLINE_O_NORMAL	0x008
-#define M_SPLINE_C_NORMAL	0x010
-#define M_SPLINE_O_INTERP	0x020
-#define M_SPLINE_C_INTERP	0x040
-#define M_TEXT_NORMAL		0x080
-#define M_TEXT_HIDDEN		0x100
-#define M_ARC			0x200
-#define M_ELLIPSE		0x400
-#define M_COMPOUND		0x800
+#define M_NONE			0x0000
+#define M_POLYLINE_POLYGON	0x0001
+#define M_POLYLINE_LINE		0x0002
+#define M_POLYLINE_BOX		0x0004	/* includes ARCBOX */
+#define M_SPLINE_O_APPROX	0x0008
+#define M_SPLINE_C_APPROX	0x0010
+#define M_SPLINE_O_INTERP	0x0020
+#define M_SPLINE_C_INTERP	0x0040
+#define M_SPLINE_O_XSPLINE      0x0080
+#define M_SPLINE_C_XSPLINE      0x0100
+#define M_TEXT_NORMAL		0x0200
+#define M_TEXT_HIDDEN		0x0400
+#define M_ARC			0x0800
+#define M_ELLIPSE		0x1000
+#define M_COMPOUND		0x2000
 
 #define M_TEXT		(M_TEXT_HIDDEN | M_TEXT_NORMAL)
-#define M_SPLINE_O	(M_SPLINE_O_NORMAL | M_SPLINE_O_INTERP)
-#define M_SPLINE_C	(M_SPLINE_C_NORMAL | M_SPLINE_C_INTERP)
-#define M_SPLINE_NORMAL (M_SPLINE_O_NORMAL | M_SPLINE_C_NORMAL)
+#define M_SPLINE_O	(M_SPLINE_O_APPROX | M_SPLINE_O_INTERP | M_SPLINE_O_XSPLINE)
+#define M_SPLINE_C	(M_SPLINE_C_APPROX | M_SPLINE_C_INTERP | M_SPLINE_C_XSPLINE)
+#define M_SPLINE_APPROX (M_SPLINE_O_APPROX | M_SPLINE_C_APPROX)
 #define M_SPLINE_INTERP (M_SPLINE_O_INTERP | M_SPLINE_C_INTERP)
-#define M_SPLINE	(M_SPLINE_NORMAL | M_SPLINE_INTERP)
+#define M_SPLINE_XSPLINE (M_SPLINE_O_XSPLINE | M_SPLINE_C_XSPLINE)
+#define M_SPLINE	(M_SPLINE_APPROX | M_SPLINE_INTERP | M_SPLINE_XSPLINE)
 #define M_POLYLINE	(M_POLYLINE_LINE | M_POLYLINE_POLYGON | M_POLYLINE_BOX)
 #define M_VARPTS_OBJECT (M_POLYLINE_LINE | M_POLYLINE_POLYGON | M_SPLINE)
 #define M_OPEN_OBJECT	(M_POLYLINE_LINE | M_SPLINE_O | M_ARC)
@@ -401,3 +437,5 @@ extern F_spline		*cur_s, *new_s, *old_s;
 extern F_compound	*cur_c, *new_c, *old_c;
 extern F_point		*first_point, *cur_point;
 extern F_linkinfo	*cur_links;
+
+#endif /* OBJECT_H */
