@@ -10,11 +10,14 @@
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
  * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.  This license includes without
- * limitation a license to do the foregoing actions under any patents of
- * the party supplying this software to the X Consortium.
+ * and/or sell copies of the Software subject to the restriction stated
+ * below, and to permit persons who receive copies from any such party to
+ * do so, with the only requirement being that this copyright notice remain
+ * intact.
+ * This license includes without limitation a license to do the foregoing
+ * actions under any patents of the party supplying this software to the 
+ * X Consortium.
+ *
  */
 
 #include "fig.h"
@@ -120,10 +123,10 @@ free_spline(list)
 free_splinestorage(s)
     F_spline	   *s;
 {
-    F_control	   *a, *b;
+    F_sfactor	   *a, *b;
 
     free_points(s->points);
-    for (a = s->controls; a != NULL; a = b) {
+    for (a = s->sfactors; a != NULL; a = b) {
 	b = a->next;
 	free((char *) a);
     }
