@@ -10,11 +10,14 @@
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
  * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.  This license includes without
- * limitation a license to do the foregoing actions under any patents of
- * the party supplying this software to the X Consortium.
+ * and/or sell copies of the Software subject to the restriction stated
+ * below, and to permit persons who receive copies from any such party to
+ * do so, with the only requirement being that this copyright notice remain
+ * intact.
+ * This license includes without limitation a license to do the foregoing
+ * actions under any patents of the party supplying this software to the 
+ * X Consortium.
+ *
  */
 
 #include "fig.h"
@@ -54,26 +57,36 @@ char *shell_protect_string(string)
     return(buf);
 }
 
-print_to_printer(printer, mag, flushleft, params)
+print_to_printer(printer, mag,  params)
     char	    printer[];
-    Boolean	    flushleft;
     float	    mag;
     char	    params[];
 {
     char	    prcmd[2*PATH_MAX+200], translator[255];
     char	    syspr[2*PATH_MAX+200];
     char	    tmpfile[PATH_MAX];
+    char	   *outfile;
 
     sprintf(tmpfile, "%s/%s%06d", TMPDIR, "xfig-print", getpid());
     warnexist = False;
     if (write_file(tmpfile))
 	return;
 
-    sprintf(translator, "fig2dev -Lps %s -P -m %f %s -n %s",
-	    flushleft ? "" : "-c" ,
-	    mag,
+    outfile = shell_protect_string(cur_filename);
+    if (!outfile || outfile[0] == '\0')
+	outfile = "NoName";
+#ifdef I18N
+    sprintf(translator, "fig2dev -Lps %s %s -P -z %s %s -m %f %s -n %s",
+	    appres.international ? appres.fig2dev_localize_option : "",
+#else
+    sprintf(translator, "fig2dev -Lps %s -P -z %s %s -m %f %s -n %s",
+#endif /* I18N */
+	    (!appres.multiple && !appres.flushleft ? "-c" : "") ,
+	    paper_sizes[appres.papersize].sname,
+	    appres.multiple ? "-M" : "",
+	    mag/100.0,
 	    appres.landscape ? "-l xxx" : "-p xxx",
-	    cur_filename);
+	    outfile);
 
     /* make the print command with no filename (it will be in stdin) */
     gen_print_cmd(syspr, "", printer, params);
@@ -84,11 +97,13 @@ print_to_printer(printer, mag, flushleft, params)
 	file_msg("Error during PRINT (check standard error output)");
     else {
 	if (emptyname(printer))
-	    put_msg("Printing figure on printer %s in %s mode ... done",
-		    printer, appres.landscape ? "LANDSCAPE" : "PORTRAIT");
+	    put_msg("Printing on default printer with %s paper size in %s mode ... done",
+		paper_sizes[appres.papersize].sname,
+		appres.landscape ? "LANDSCAPE" : "PORTRAIT");
 	else
-	    put_msg("Printing figure on printer %s in %s mode ... done",
-		    printer, appres.landscape ? "LANDSCAPE" : "PORTRAIT");
+	    put_msg("Printing on \"%s\" with %s paper size in %s mode ... done",
+		printer, paper_sizes[appres.papersize].sname,
+		appres.landscape ? "LANDSCAPE" : "PORTRAIT");
     }
     unlink(tmpfile);
 }
@@ -109,7 +124,8 @@ gen_print_cmd(cmd,file,printer,pr_params)
 		pr_params,
 		shell_protect_string(file));
 #endif
-	put_msg("Printing figure on default printer in %s mode ...     ",
+	put_msg("Printing on default printer with %s paper size in %s mode ...     ",
+		paper_sizes[appres.papersize].sname,
 		appres.landscape ? "LANDSCAPE" : "PORTRAIT");
     } else {
 #if (defined(SYSV) || defined(SVR4)) && !defined(BSDLPR)
@@ -123,18 +139,18 @@ gen_print_cmd(cmd,file,printer,pr_params)
 		printer,
 		shell_protect_string(file));
 #endif
-	put_msg("Printing figure on printer %s in %s mode ...     ",
-		printer, appres.landscape ? "LANDSCAPE" : "PORTRAIT");
+	put_msg("Printing on \"%s\" with %s paper size in %s mode ...     ",
+		printer, paper_sizes[appres.papersize].sname,
+		appres.landscape ? "LANDSCAPE" : "PORTRAIT");
     }
     app_flush();		/* make sure message gets displayed */
 }
 
 /* xoff and yoff are in fig2dev print units (1/72 inch) */
 
-print_to_file(file, lang, mag, flushleft, xoff, yoff)
+print_to_file(file, lang, mag, xoff, yoff)
     char	   *file, *lang;
     float	    mag;
-    Boolean	    flushleft;
     int		    xoff, yoff;
 {
     char	    prcmd[2*PATH_MAX+200];
@@ -154,46 +170,68 @@ print_to_file(file, lang, mag, flushleft, xoff, yoff)
 	return (1);
     outfile = shell_protect_string(file);
 
-    put_msg("Exporting figure to file \"%s\" in %s mode ...     ",
+    put_msg("Exporting to file \"%s\" in %s mode ...     ",
 	    file, appres.landscape ? "LANDSCAPE" : "PORTRAIT");
     app_flush();		/* make sure message gets displayed */
 
     if (!strcmp(lang, "ps"))
-	sprintf(prcmd, "fig2dev -Lps %s -P -m %f %s -n %s -x %d -y %d %s %s", 
-		flushleft ? "" : "-c" ,
-		mag, appres.landscape ? "-l xxx" : "-p xxx", cur_filename,
+#ifdef I18N
+	sprintf(prcmd, "fig2dev -Lps %s %s -P -z %s %s -m %f %s -n %s -x %d -y %d %s %s", 
+		appres.international ? appres.fig2dev_localize_option : "",
+#else
+	sprintf(prcmd, "fig2dev -Lps %s -P -z %s %s -m %f %s -n %s -x %d -y %d %s %s", 
+#endif  /* I18N */
+		(!appres.multiple && !appres.flushleft ? "-c" : "") ,
+		paper_sizes[appres.papersize].sname,
+		appres.multiple ? "-M" : "",
+		mag/100.0, appres.landscape ? "-l xxx" : "-p xxx", outfile,
 		xoff, yoff,
 		tmp_fig_file, outfile);
     else if (!strcmp(lang, "eps"))
-	sprintf(prcmd, "fig2dev -Lps -m %f %s -n %s %s %s",
-		mag, appres.landscape ? "-l xxx" : "-p xxx",
-		cur_filename, tmp_fig_file, outfile);
-    else if (!strcmp(lang, "ibmgl"))
+#ifdef I18N
+	sprintf(prcmd, "fig2dev -Lps %s -z %s -m %f %s -n %s %s %s",
+		appres.international ? appres.fig2dev_localize_option : "",
+#else
+	sprintf(prcmd, "fig2dev -Lps -z %s -m %f %s -n %s %s %s",
+#endif  /* I18N */
+		paper_sizes[appres.papersize].sname,
+		mag/100.0, appres.landscape ? "-l xxx" : "-p xxx",
+		outfile, tmp_fig_file, outfile);
+    else if (!strcmp(lang, "hpl"))
 	sprintf(prcmd, "fig2dev -Libmgl -m %f %s %s %s",
-		mag, appres.landscape ? "" : "-P", tmp_fig_file,
+		mag/100.0, appres.landscape ? "" : "-P", tmp_fig_file,
 		outfile);
-    else if (!strcmp(lang, "pstex_t")) {
-	/* make it automatically input the postscript part */
-	strcpy(tmp_name, file);
-	tlen = strlen(tmp_name);
-	if (tlen > 2) {
-	    if (tmp_name[tlen-1] == 't' && tmp_name[tlen-2] == '_')
-		tmp_name[tlen-2] = '\0';
-	    else
-		tmp_name[0] = '\0';
-	} else
-	    tmp_name[0] = '\0';
+    else if (!strcmp(lang, "pstex")) {
+	/* do both PostScript part and text part */
+	/* first the postscript part */
+	sprintf(prcmd, "fig2dev -L%s -m %f -n %s %s %s", lang,
+		mag/100.0, outfile, tmp_fig_file, outfile);
+	if (appres.DEBUG)
+	    fprintf(stderr,"execing: %s\n",prcmd);
+	if (system(prcmd) != 0)
+	    file_msg("Error during EXPORT of PostScript part (check standard error output)");
+	/* now the text part */
+	/* add "_t" to the output filename and put in tmp_name */
+	strcpy(tmp_name,outfile);
+	strcat(tmp_name,"_t");
+	/* make it automatically input the postscript part (-p option) */
 	sprintf(prcmd, "fig2dev -Lpstex_t -p %s -m %f %s %s",
-		tmp_name, mag, tmp_fig_file, outfile);
+		outfile, mag/100.0, tmp_fig_file, tmp_name);
+    } else if (!strcmp(lang, "jpeg")) {
+	/* set the image quality for JPEG export */
+	sprintf(prcmd, "fig2dev -L%s -q %d -m %f %s %s", lang,
+		appres.jpeg_quality, mag/100.0, tmp_fig_file, outfile);
     } else
 	sprintf(prcmd, "fig2dev -L%s -m %f %s %s", lang,
-		mag, tmp_fig_file, outfile);
+		mag/100.0, tmp_fig_file, outfile);
+
+    /* now execute fig2dev */
     if (appres.DEBUG)
 	fprintf(stderr,"execing: %s\n",prcmd);
     if (system(prcmd) != 0)
 	file_msg("Error during EXPORT (check standard error output)");
     else
-	put_msg("Exporting figure to file \"%s\" in %s mode ... done",
+	put_msg("Exporting to file \"%s\" in %s mode ... done",
 		file, appres.landscape ? "LANDSCAPE" : "PORTRAIT");
 
     unlink(tmp_fig_file);
