@@ -10,11 +10,14 @@
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
  * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.  This license includes without
- * limitation a license to do the foregoing actions under any patents of
- * the party supplying this software to the X Consortium.
+ * and/or sell copies of the Software subject to the restriction stated
+ * below, and to permit persons who receive copies from any such party to
+ * do so, with the only requirement being that this copyright notice remain
+ * intact.
+ * This license includes without limitation a license to do the foregoing
+ * actions under any patents of the party supplying this software to the 
+ * X Consortium.
+ *
  */
 
 #include "fig.h"
@@ -544,7 +547,7 @@ next_spline_point_found(x, y, tol, p, q, shift)
 	    n++;
 	    *p = NULL;
 	    for (*q = s->points; *q != NULL; *p = *q, *q = (*q)->next) {
-		if (abs((*q)->x - x) <= tol && abs((*q)->y - y) <= tol)
+		if ((abs((*q)->x - x) <= tol) && (abs((*q)->y - y) <= tol))
 		    return (1);
 	    }
 	}
@@ -889,3 +892,26 @@ compound_point_search(x, y, tol, cx, cy, fx, fy)
     }
     return (NULL);
 }
+
+
+
+F_spline   *
+get_spline_point(x, y, p, q)
+    int		    x, y;
+    F_point	  **p, **q;
+{
+    F_spline *spline;
+    spline = last_spline(objects.splines);
+    for (; spline != NULL; spline = prev_spline(objects.splines, spline))
+	if (validspline_in_mask(spline)) {
+	    n++;
+	    *p = NULL;
+	    for (*q = spline->points; *q != NULL; *p = *q, *q = (*q)->next) {
+		if ((abs((*q)->x - x) <= TOLERANCE) && 
+		    (abs((*q)->y - y) <= TOLERANCE))
+		    return spline;
+	    }
+	}
+    return (NULL);
+}
+
