@@ -10,11 +10,14 @@
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
  * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.  This license includes without
- * limitation a license to do the foregoing actions under any patents of
- * the party supplying this software to the X Consortium.
+ * and/or sell copies of the Software subject to the restriction stated
+ * below, and to permit persons who receive copies from any such party to
+ * do so, with the only requirement being that this copyright notice remain
+ * intact.
+ * This license includes without limitation a license to do the foregoing
+ * actions under any patents of the party supplying this software to the 
+ * X Consortium.
+ *
  */
 
 /*********************** IMPORTS ************************/
@@ -25,12 +28,14 @@
 #include "mode.h"
 #include "paintop.h"
 #include <X11/keysym.h>
+#include "e_edit.h"
 #include "u_bound.h"
 #include "w_canvas.h"
 #include "w_mousefun.h"
 #include "w_setup.h"
 #include "w_util.h"
 #include "w_zoom.h"
+
 #ifndef SYSV
 #include <sys/time.h>
 #endif
@@ -40,6 +45,11 @@ extern		erase_rulermark();
 extern		erase_objecthighlight();
 extern		char_handler();
 
+#ifdef I18N
+extern XIC xim_ic;
+extern i18n_char_handler();
+#endif
+
 /************** LOCAL STRUCTURE ***************/
 
 typedef struct _CompKey CompKey;
@@ -70,7 +80,11 @@ String		local_translations = "";
 /*********************** LOCAL ************************/
 
 static CompKey *allCompKey = NULL;
+#ifdef I18N
+extern		canvas_selected();
+#else
 static		canvas_selected();
+#endif
 static unsigned char getComposeKey();
 static		readComposeKey();
 
@@ -128,8 +142,9 @@ XtActionsRec	canvas_actions[] =
 /* need the ~Meta for the EventCanv action so that the accelerators still work 
    during text input */
 static String	canvas_translations =
-"<Motion>:EventCanv()\n\
+   "<Motion>:EventCanv()\n\
     Any<BtnDown>:EventCanv()\n\
+    Any<BtnUp>:EventCanv()\n\
     <Key>F18: PasteCanv()\n\
     <EnterWindow>:EnterCanv()\n\
     <LeaveWindow>:LeaveCanv()EraseRulerMark()\n\
@@ -140,9 +155,7 @@ static String	canvas_translations =
 init_canvas(tool)
     Widget	   tool;
 {
-    XColor	   fixcolors[2];
-
-    DeclareArgs(10);
+    DeclareArgs(12);
 
     FirstArg(XtNlabel, "");
     NextArg(XtNwidth, CANVAS_WD);
@@ -157,28 +170,6 @@ init_canvas(tool)
 
     canvas_sw = XtCreateWidget("canvas", labelWidgetClass, tool,
 			       Args, ArgCount);
-
-    FirstArg(XtNforeground, &x_fg_color.pixel);
-    NextArg(XtNbackground, &x_bg_color.pixel);
-    GetValues(canvas_sw);
-
-    /*
-     * get the RGB values for recolor cursor use -- may want to have cursor
-     * color resource
-     */
-    fixcolors[0] = x_fg_color;
-    fixcolors[1] = x_bg_color;
-    XQueryColors(tool_d, tool_cm, fixcolors, 2);
-    x_fg_color = fixcolors[0];
-    x_bg_color = fixcolors[1];
-
-    /* now fix the global GC */
-    XSetState(tool_d, gc, x_fg_color.pixel, x_bg_color.pixel, GXcopy,
-	      AllPlanes);
-
-    /* and recolor the cursors */
-    recolor_cursors();
-
     canvas_leftbut_proc = null_proc;
     canvas_middlebut_proc = null_proc;
     canvas_rightbut_proc = null_proc;
@@ -201,7 +192,9 @@ setup_canvas()
     reset_clip_window();
 }
 
+#ifndef I18N
 static
+#endif
 canvas_selected(tool, event, params, nparams)
     Widget	    tool;
     XButtonEvent   *event;
@@ -214,6 +207,9 @@ canvas_selected(tool, event, params, nparams)
     char	    buf[1];
     XButtonPressedEvent *be = (XButtonPressedEvent *) event;
     XKeyPressedEvent *ke = (XKeyPressedEvent *) event;
+    Window	    rw, cw;
+    int		    rx, ry, cx, cy;
+    unsigned int    mask;
 
     static char	    compose_buf[2];
     static char	    compose_key = 0;
@@ -234,28 +230,23 @@ canvas_selected(tool, event, params, nparams)
 	sx = x;
 	sy = y;
 #else
-	{
-	    Window	    rw, cw;
-	    int		    rx, ry, cx, cy;
-	    unsigned int    mask;
-
-	    XQueryPointer(event->display, event->window,
+	XQueryPointer(event->display, event->window,
 			  &rw, &cw,
 			  &rx, &ry,
 			  &cx, &cy,
 			  &mask);
-	    cx = BACKX(cx);
-	    cy = BACKY(cy);
+	cx = BACKX(cx);
+	cy = BACKY(cy);
 
-	    /* perform appropriate rounding if necessary */
-	    round_coords(cx, cy);
+	/* perform appropriate rounding if necessary */
+	round_coords(cx, cy);
 
-	    if (cx == sx && cy == sy)
+	if (cx == sx && cy == sy)
 		break;
-	    x = sx = cx;	/* these are zoomed */
-	    y = sy = cy;	/* coordinates!	    */
-	}
+	x = sx = cx;	/* these are zoomed */
+	y = sy = cy;	/* coordinates!	    */
 #endif /* SMOOTHMOTION */
+
 	set_rulermark(x, y);
 	(*canvas_locmove_proc) (x, y);
 	break;
@@ -270,11 +261,19 @@ canvas_selected(tool, event, params, nparams)
 	    be->state &= ~Mod1Mask;
 	}
 
-	/* call interactive zoom function if control key is pressed */
-	if (be->state & ControlMask) {
+	/* call interactive zoom function when only control key pressed */
+	if (!zoom_in_progress && ((be->state & ControlMask) && !(be->state & ShiftMask))) {
 	    zoom_selected(x, y, be->button);
 	    break;
 	}
+
+        /* edit shape factor when pressing control & shift keys in edit mode */
+	if ((be->state & ControlMask && be->state & ShiftMask) &&
+	    cur_mode >= FIRST_EDIT_MODE) {
+		change_sfactor(x, y, be->button);
+		break;
+	}
+
 	/* perform appropriate rounding if necessary */
 	round_coords(x, y);
 
@@ -286,20 +285,45 @@ canvas_selected(tool, event, params, nparams)
 	    (*canvas_rightbut_proc) (x, y, be->state & ShiftMask);
 	break;
     case KeyPress:
+    case KeyRelease:
+	XQueryPointer(event->display, event->window, 
+			&rw, &cw, &rx, &ry, &cx, &cy, &mask);
 	/* we might want to check action_on */
 	/* if arrow keys are pressed, pan */
 	key = XLookupKeysym(ke, 0);
-	if (key == XK_Left ||
-	    key == XK_Right ||
-	    key == XK_Up ||
+
+	/* do the mouse function stuff first */
+	if (zoom_in_progress) {
+		set_temp_cursor(magnify_cursor);
+		draw_mousefun("final point", "", "cancel");
+	} else if (mask & ControlMask) {
+		if (mask & ShiftMask) {
+		  reset_cursor();
+		  if (cur_mode >= FIRST_EDIT_MODE)
+		    draw_mousefun("More approx", "Cycle shapes", "More interp");
+		  else
+		    draw_shift_mousefun_canvas();
+		} else {  /* show control-key action */
+		  set_temp_cursor(magnify_cursor);
+		  draw_mousefun("Zoom area", "Pan to origin", "Unzoom");
+		}
+	} else if (mask & ShiftMask) {
+		reset_cursor();
+		if (cur_mode >= FIRST_EDIT_MODE)
+		  draw_mousefun("Locate object", "Locate object", "");
+		else
+		  draw_shift_mousefun_canvas();
+	} else {
+		reset_cursor();
+		draw_mousefun_canvas();
+	}
+
+	if (event->type == KeyPress) {
+	  if (key == XK_Up ||
 	    key == XK_Down ||
-	    key == XK_Home ||
-	    key == XK_Multi_key ||
-	    key == XK_Meta_L ||
-	    key == XK_Meta_R ||
-	    key == XK_Alt_L ||
-	    key == XK_Alt_R ||
-	    key == XK_Escape ) {
+	    ((key == XK_Left ||    /* *don't* process the following if in text input mode */
+	      key == XK_Right ||
+	      key == XK_Home) && (!action_on || cur_mode != F_TEXT))) {
 	        switch (key) {
 		    case XK_Left:
 			pan_left(event->state&ShiftMask);
@@ -316,47 +340,78 @@ canvas_selected(tool, event, params, nparams)
 		    case XK_Home:
 			pan_origin();
 			break;
-		    case XK_Multi_key:
-		    case XK_Meta_L:
-		    case XK_Meta_R:
-		    case XK_Alt_L:
-		    case XK_Alt_R:
-		    case XK_Escape:
+		}
+	} else if 
+	     ((key == XK_Multi_key || /* process the following *only* if in text input mode */
+	      key == XK_Meta_L ||
+	      key == XK_Meta_R ||
+	      key == XK_Alt_L ||
+	      key == XK_Alt_R ||
+	      key == XK_Escape) && action_on && cur_mode == F_TEXT) {
 			compose_key = 1;
 			break;
-		}
-	} else if (key == XK_Control_L || key == XK_Control_R) { 
-		/* show the control-key actions */
-		set_temp_cursor(magnify_cursor);
-		draw_mousefun("Zoom area", "Pan to origin", "Unzoom");
-	} else if (key == XK_Shift_L || key == XK_Shift_R) {
-		/* show the shift-key function, but only if an edit mode */
-		if (cur_mode >= FIRST_EDIT_MODE)
-		    draw_mousefun("Locate object", "Locate object", "");
-		else
-		    draw_shift_mousefun_canvas();
 	} else {
 	    if (canvas_kbd_proc != null_proc) {
-		switch (compose_key) {
-		case 0:
-		    if (XLookupString(ke, buf, sizeof(buf), NULL, NULL) > 0)
-			(*canvas_kbd_proc) ((unsigned char) buf[0]);
-		    break;
-		case 1:
-		    if (XLookupString(ke, &compose_buf[0], 1, NULL, NULL) > 0)
-			compose_key = 2;
-		    break;
-		case 2:
-		    if (XLookupString(ke, &compose_buf[1], 1, NULL, NULL) > 0) {
-			if ((c = getComposeKey(compose_buf)) != '\0')
-			    (*canvas_kbd_proc) (c);
-			else {
-			    (*canvas_kbd_proc) ((unsigned char) compose_buf[0]);
-			    (*canvas_kbd_proc) ((unsigned char) compose_buf[1]);
-			}
-			compose_key = 0;
-		    }
-		    break;
+		if (key == XK_Left || key == XK_Right || key == XK_Home || key == XK_End) {
+			(*canvas_kbd_proc) ((unsigned char) 0, key);
+		    compose_key = 0;	/* in case Meta was followed with cursor movement */
+		} else {
+		    switch (compose_key) {
+			case 0:
+#ifdef I18N
+			    if (xim_ic != NULL) {
+			      static int lbuf_size = 0;
+			      static char *lbuf = NULL;
+			      KeySym key_sym;
+			      Status status;
+			      int i, len;
+
+			       if (lbuf == NULL) {
+				 lbuf_size = 100;
+				 lbuf = malloc(lbuf_size + 1);
+			       }
+			       len = XmbLookupString(xim_ic, ke, lbuf, lbuf_size,
+						     &key_sym, &status);
+			       if (status == XBufferOverflow) {
+				 lbuf_size = len;
+				 lbuf = realloc(lbuf, lbuf_size + 1);
+				 len = XmbLookupString(xim_ic, ke, lbuf, lbuf_size,
+						       &key_sym, &status);
+			       }
+			       if (status == XBufferOverflow) {
+				 fprintf(stderr, "xfig: buffer overflow (XmbLookupString)\n");
+			       }
+			       lbuf[len] = '\0';
+			       if (0 < len) {
+				 if (2 <= len && canvas_kbd_proc == char_handler) {
+				   i18n_char_handler(lbuf);
+				 } else {
+				   for (i = 0; i < len; i++) {
+				     (*canvas_kbd_proc) (lbuf[i], (KeySym) 0);
+				   }
+				 }
+			       }
+			    } else
+#endif  /* I18N */
+			    if (XLookupString(ke, buf, sizeof(buf), NULL, NULL) > 0)
+				(*canvas_kbd_proc) (buf[0], (KeySym) 0);
+			    break;
+			case 1:
+			    if (XLookupString(ke, &compose_buf[0], 1, NULL, NULL) > 0)
+				compose_key = 2;
+			    break;
+			case 2:
+			    if (XLookupString(ke, &compose_buf[1], 1, NULL, NULL) > 0) {
+				if ((c = getComposeKey(compose_buf)) != '\0')
+				    (*canvas_kbd_proc) (c, (KeySym) 0);
+				else {
+				    (*canvas_kbd_proc) ((unsigned char) compose_buf[0], (KeySym) 0);
+				    (*canvas_kbd_proc) ((unsigned char) compose_buf[1], (KeySym) 0);
+				}
+				compose_key = 0;
+			    }
+			    break;
+		    } /* switch */
 		}
 	    } else {
 		/* Be cheeky... we aren't going to do anything, so pass the
@@ -367,20 +422,9 @@ canvas_selected(tool, event, params, nparams)
 		ke->subwindow = 0;
 		XPutBackEvent(ke->display,(XEvent *)ke);
 	    }
+	  }
+	  break;
 	}
-	break;
-
-    case KeyRelease:
-	/* if user is releasing Control or Shift keys redisplay original function */
-	key = XLookupKeysym(ke, 0);
-	    if (key == XK_Control_L || key == XK_Control_R ||
-		key == XK_Shift_L || key == XK_Shift_R)
-		    draw_mousefun_canvas();
-	    /* if releasing Control key and not in zoom mode, reset from 
-		magnifying glass to previous cursor */
-	    if ((key == XK_Control_L || key == XK_Control_R) && !zoom_in_progress)
-		reset_cursor();
-	break;
     }
 }
 
@@ -399,7 +443,8 @@ clear_region(xmin, ymin, xmax, ymax)
 
 static void get_canvas_clipboard();
 
-static void canvas_paste(w, paste_event)
+static void
+canvas_paste(w, paste_event)
 Widget w;
 XKeyEvent *paste_event;
 {
@@ -426,12 +471,12 @@ XtPointer buf;
 unsigned long *length;
 int *format;
 {
-	char *c;
+	unsigned char *c;
 	int i;
 
-	c = buf;
+	c = (unsigned char *) buf;
 	for (i=0; i<*length; i++) {
-           canvas_kbd_proc(*c);
+           canvas_kbd_proc(*c, (KeySym) 0);
            c++;
 	}
 	XtFree(buf);
