@@ -4,6 +4,9 @@
  * Copyright (c) 1994 by Brian V. Smith
  * Parts Copyright 1990,1992 Richard Hesketh
  *          Computing Lab. University of Kent at Canterbury, UK
+ * Pixel Grab color lookup Copyright 1993, David Koblas (koblas@netcom.com)
+ * and Copyright 1995, 1996 Torsten Martinsen (bullestock@dk-online.dk)
+ * (full copyright and permission notice appears above code)
  *
  * The X Consortium, and any party obtaining a copy of these files from
  * the X Consortium, directly or indirectly, is granted, free of charge, a
@@ -11,11 +14,16 @@
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
  * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.  This license includes without
- * limitation a license to do the foregoing actions under any patents of
- * the party supplying this software to the X Consortium.
+ * and/or sell copies of the Software subject to the restriction stated
+ * below, and to permit persons who receive copies from any such party to
+ * do so, with the only requirement being that this copyright notice remain
+ * intact.
+ * This license includes without limitation a license to do the foregoing
+ * actions under any patents of the party supplying this software to the 
+ * X Consortium.
+ *
+ *
+ ****************************************************************************
  *
  * Parts of this work were extracted from Richard Hesketh's xcoloredit
  * client. Here is the copyright notice which must remain intact:
@@ -97,11 +105,15 @@ static void set_std_color();
 static void add_color();
 static void del_color();
 static void undel_color();
+static void lookup_color();
 
 /* some local procedures */
 
 static int WhichButton();
 static Boolean color_used();
+static void DoGrabPixel();
+static void doGrab();
+static void xyToWindowCmap();
 
 #define S_RED    1
 #define S_GREEN  2
@@ -114,15 +126,17 @@ static Boolean color_used();
 /* width/height and space between the scrollbars */
 #define SCROLL_W	23
 #define SCROLL_H	120
-#define SCROLL_SP	5
+#define SCROLL_SP	4
 
 /* width/height of the standard color cells */
-#define STD_COL_W	30
+#define STD_COL_W	31
 #define STD_COL_H	20
 
 /* width/height of the user color cells */
 #define USR_COL_W	30
 #define USR_COL_H	20
+/* spacing between cells */
+#define USR_COL_SP	4
 
 /* thickness (height) of the scrollbar (fraction of total) */
 #define	THUMB_H		0.04
@@ -168,7 +182,7 @@ static Widget	redScroll, greenScroll, blueScroll, lockedScroll;
 static Widget	hueScroll, satScroll, valScroll;
 static Widget	hueLabel, satLabel, valLabel;
 static Widget	ok_button;
-static Widget	addColor, delColor, undelColor;
+static Widget	addColor, delColor, undelColor, lookupColor;
 static Widget	userLabel, userForm, userViewport, userBox;
 static Widget	colorMemory[MAX_USR_COLS];
 
@@ -235,6 +249,38 @@ static String user_color_translations =
 static String triple_translations =
 	"<Key>Return: update_from_triple()\n";
 
+YStoreColor(colormap,color)
+Colormap colormap;
+XColor *color;
+{
+	switch( tool_vclass ){
+	    case GrayScale:
+	    case PseudoColor:
+		XStoreColor(tool_d,colormap,color);
+		break;
+	    case StaticGray:
+	    case StaticColor:
+	    case DirectColor:
+	    case TrueColor:
+		XAllocColor(tool_d,colormap,color);
+		break;
+	    default:
+		if (appres.DEBUG)
+		    fprintf(stderr,"Unknown Visual Class\n");
+	}
+}
+
+YStoreColors(colormap,color,ncolors)
+Colormap colormap;
+XColor color[];
+int ncolors;
+{
+	int		 i;
+
+	for( i=0; i<ncolors; i++ )
+	    YStoreColor(colormap,&color[i]);
+}
+
 create_color_panel(form,label,cancel,isw)
 Widget		 form, label, cancel;
 ind_sw_info	*isw;
@@ -260,7 +306,7 @@ ind_sw_info	*isw;
 
 	if (all_colors_available) {
 	    /* allocate two colorcells for the mixed fill and pen color widgets */
-	    if (!alloc_color(pixels,2)) {
+	    if (!alloc_color_cells(pixels,2)) {
 		file_msg("Can't allocate necessary colorcells, can't popup color panel");
 		return;
 	    }
@@ -437,21 +483,19 @@ ind_sw_info	*isw;
 
 	/* make a scrollable viewport widget to contain the color memory buttons */
 
-	FirstArg(XtNallowHoriz, True);
-	NextArg(XtNwidth, isw->sw_per_row*(STD_COL_W+5)+1);
-	NextArg(XtNheight, 47);
+	FirstArg(XtNallowVert, True);
+	NextArg(XtNwidth, isw->sw_per_row*(STD_COL_W+3));
+	NextArg(XtNheight, (USR_COL_H+USR_COL_SP)*3+22);  /* the 22 is for the scrollbar */
 	NextArg(XtNborderWidth, 1);
-	NextArg(XtNuseBottom, True);
 	NextArg(XtNforceBars, True);
 
 	userViewport = XtCreateManagedWidget("userViewport", viewportWidgetClass, 
 			userForm, Args, ArgCount);
 
-	FirstArg(XtNheight, 47);
-	NextArg(XtNhSpace, 5);
+	FirstArg(XtNhSpace, USR_COL_SP);		/* spacing between cells */
 	NextArg(XtNresizable, True);
 	NextArg(XtNborderWidth, 0);
-	NextArg(XtNorientation, XtorientHorizontal);	/* expand horizontally */
+	NextArg(XtNorientation, XtorientVertical);
 
 	userBox = XtCreateManagedWidget("userBox", boxWidgetClass, userViewport,
 			       Args, ArgCount);
@@ -461,10 +505,19 @@ ind_sw_info	*isw;
 	    if (colorFree[i])
 		continue;
 	    sprintf(str,"%d", i+NUM_STD_COLS);
+	    
+	    /* pick contrasting color for label in cell */
+	    XQueryColor(tool_d, tool_cm, &user_colors[i]);
+	    if ((0.30 * user_colors[i].red +
+		 0.59 * user_colors[i].green +
+		 0.11 * user_colors[i].blue) < 0.5 * (255 << 8))
+			form_fg = colors[WHITE];
+	    else
+			form_fg = colors[BLACK];
 	    colorMemory[i] = XtVaCreateManagedWidget("colorMemory",
 				labelWidgetClass, userBox,
 				XtNlabel, str, 
-				XtNforeground, x_bg_color.pixel,
+				XtNforeground, form_fg,
 				XtNbackground, (all_colors_available? 
 					user_colors[i].pixel: x_fg_color.pixel),
 				XtNwidth, USR_COL_W, XtNheight, USR_COL_H,
@@ -489,22 +542,30 @@ ind_sw_info	*isw;
 
 	/* now the add/delete color buttons */
 
-	FirstArg(XtNlabel, "Add Color");
+	FirstArg(XtNlabel, "Add");
 	NextArg(XtNfromVert, userViewport);
 	addColor = XtCreateManagedWidget("addColor", commandWidgetClass,
 					       userForm, Args, ArgCount);
 	XtAddEventHandler(addColor, ButtonReleaseMask, (Boolean) 0,
 			  (XtEventHandler) add_color, (XtPointer) 0);
 
-	FirstArg(XtNlabel, "Del Color");
+	FirstArg(XtNlabel, "Lookup");
 	NextArg(XtNfromHoriz, addColor);
 	NextArg(XtNfromVert, userViewport);
+	lookupColor = XtCreateManagedWidget("lookupColor", commandWidgetClass,
+					       userForm, Args, ArgCount);
+	XtAddEventHandler(lookupColor, ButtonReleaseMask, (Boolean) 0,
+			  (XtEventHandler) lookup_color, (XtPointer) 0);
+
+	FirstArg(XtNlabel, "Delete");
+	NextArg(XtNfromHoriz, lookupColor);
+	NextArg(XtNfromVert, userViewport);
 	delColor = XtCreateManagedWidget("delColor", commandWidgetClass,
 					       userForm, Args, ArgCount);
 	XtAddEventHandler(delColor, ButtonReleaseMask, (Boolean) 0,
 			  (XtEventHandler) del_color, (XtPointer) 0);
 
-	FirstArg(XtNlabel, "UnDel Color");
+	FirstArg(XtNlabel, "UnDelete");
 	NextArg(XtNfromHoriz, delColor);
 	NextArg(XtNfromVert, userViewport);
 	undelColor = XtCreateManagedWidget("undelColor", commandWidgetClass,
@@ -695,7 +756,7 @@ ind_sw_info	*isw;
 
 	/* get the name of the scrollbar in the user color viewport so we can 
 	   make it solid instead of the default grey pixmap */
-	sb = XtNameToWidget(userViewport, "horizontal");
+	sb = XtNameToWidget(userViewport, "vertical");
 	FirstArg(XtNthumb, None);
 	SetValues(sb);
 
@@ -763,7 +824,10 @@ restore_mixed_colors()
 	}
 	mixed_color[0].pixel = save0;
 	mixed_color[1].pixel = save1;
-	XStoreColors(tool_d, tool_cm, mixed_color, 2);
+	/* now change the background of the widgets if StaticGray, StaticColor,
+	   DirectColor or TrueColor visuals */
+	set_mixed_color(0);
+	set_mixed_color(1);
 
 	/* set the scrollbars to the initial mixed colour values */
 	do_change = False;
@@ -781,6 +845,74 @@ restore_mixed_colors()
 	edit_fill = save_edit;
 }
 
+/*
+   For the StaticGray, StaticColor, DirectColor and TrueColor visual classes
+   change the background color of the mixed color widget specified by which
+   (0=pen, 1=fill).  For the other classes, we changed the color for the
+   allocated pixel when we called YStoreColor().
+*/
+
+set_mixed_color(which)
+int	which;
+{
+	if( tool_vclass == StaticGray ||
+	    tool_vclass == StaticColor ||
+	    tool_vclass == DirectColor ||
+	    tool_vclass == TrueColor) {
+		XAllocColor(tool_d, tool_cm, &mixed_color[which]);
+		if (colorMemory[which]) {
+		    FirstArg(XtNbackground, mixed_color[which].pixel);
+		    SetValues(mixedColor[which]);
+		}
+	} else {
+		XStoreColor(tool_d, tool_cm, &mixed_color[which]);
+	}
+	/* now make contrasting color for the label */
+	if (colorMemory[which]) {
+	    pick_contrast(user_colors[which],mixedColor[which]);
+	}
+}
+
+/* This is similar to set_mixed_color() except it is for the user colors */
+
+set_user_color(which)
+int	which;
+{
+	if( tool_vclass == StaticGray ||
+	    tool_vclass == StaticColor ||
+	    tool_vclass == DirectColor ||
+	    tool_vclass == TrueColor) {
+		XAllocColor(tool_d, tool_cm, &user_colors[which]);
+		/* update colors pixel value */
+		colors[which+NUM_STD_COLS] = user_colors[which].pixel;
+		if (colorMemory[which]) {
+		    FirstArg(XtNbackground, user_colors[which].pixel);
+		    SetValues(colorMemory[which]);
+		}
+	} else {
+		XStoreColor(tool_d, tool_cm, &user_colors[which]);
+	}
+	/* make the label in a contrasting color */
+	if (colorMemory[which]) {
+	    pick_contrast(user_colors[which],colorMemory[which]);
+	}
+}
+
+pick_contrast(color,widget)
+XColor	color;
+Widget	widget;
+{
+    Pixel cell_fg;
+    if ((0.30 * color.red +
+	 0.59 * color.green +
+	 0.11 * color.blue) < 0.5 * (255 << 8))
+	    cell_fg = colors[WHITE];
+    else
+	    cell_fg = colors[BLACK];
+    FirstArg(XtNforeground, cell_fg);
+    SetValues(widget);
+}
+
 /* change the label of the mixedColor widget[i] to the name of the color */
 /* also set the foreground color to contrast the background */
 
@@ -788,7 +920,7 @@ set_mixed_name(i,col)
 int	i,col;
 {
     int	fore;
-    char	buf[5];
+    char	buf[10];
 
     if (col < NUM_STD_COLS) {
 	FirstArg(XtNlabel, colorNames[col+1].name)
@@ -827,153 +959,14 @@ XtPointer closure, ptr;
 {
 	/* deselect any cell currently selected */
 	if (current_memory >= 0)
-	    draw_a_dotted_box(colorMemory[current_memory], unboxedGC);
+	    draw_a_box(colorMemory[current_memory], unboxedGC);
 	/* add another widget to the user color panel */
 	if ((current_memory = add_color_cell(False, 0, 0, 0, 0)) == -1)	/* using black */
 		put_msg("No more user colors allowed");
-	draw_a_dotted_box(colorMemory[current_memory], unboxedGC);
+	draw_a_box(colorMemory[current_memory], unboxedGC);
 	modified[edit_fill] = True;
-}
-
-/* add a widget to the user color list with color r,g,b */
-/* call with use_exist true if you wish to allocate cell <indx> explicitly */
-/* also increment num_usr_cols if we add a colorcell beyond the current number */
-/* return the colorcell number (0..MAX_USR_COLS-1) */
-/* return -1 if MAX_USR_COLS already used */
-
-int
-add_color_cell(use_exist, indx, r, g, b)
-Boolean	use_exist;
-int	indx;
-int	r,g,b;
-{
-	int	i;
-	char	labl[5];
-	Boolean	new;
-
-	if (all_colors_available) {
-	    /* try to get a colorcell */
-	    if (!alloc_color(pixels,1)) {
-		put_msg("Can't allocate user color, not enough colorcells");
-		return 0;
-	    }
-	}
-	if (!use_exist) {
-		/* first look for color cell available in the middle */
-		new = False;
-		for (i=0; i<num_usr_cols; i++)
-		    if (colorFree[i])
-			break;
-		indx = i;
-	}
-
-	/* if a space is free but there was never a color there, must create one */
-	if (indx<num_usr_cols && colorMemory[indx]==0)
-	    new = True;
-
-	/* if not, increment num_usr_cols */
-	if (indx>= num_usr_cols) {
-	    if (num_usr_cols >= MAX_USR_COLS-1)
-		return -1;
-	    if (use_exist)
-		num_usr_cols = indx+1;
-	    else
-		num_usr_cols++;
-	    new = True;
-	}
-
-	user_colors[indx].red = r*256;
-	user_colors[indx].green = g*256;
-	user_colors[indx].blue = b*256;
-	user_colors[indx].flags = DoRed|DoGreen|DoBlue;
-	user_colors[indx].pixel = colors[NUM_STD_COLS+indx] = pixels[0];
-
-	if (all_colors_available)
-	    XStoreColor(tool_d, tool_cm, &user_colors[indx]);
-
-	colorUsed[indx] = False;
-	colorFree[indx] = False;
-
-	/* if the color popup has been created create the widgets */
-	if (pen_color_button->panel) {
-	    if (new) {
-		/* put the user color number in all the time */
-		sprintf(labl,"%d", indx+NUM_STD_COLS);
-		colorMemory[indx] = XtVaCreateManagedWidget("colorMemory",
-			labelWidgetClass, userBox,
-			XtNlabel, labl, 
-			XtNforeground, x_bg_color.pixel,
-			XtNbackground, (all_colors_available? 
-				user_colors[indx].pixel: x_fg_color.pixel),
-			XtNwidth, USR_COL_W, XtNheight, USR_COL_H,
-			XtNborderWidth, 1, NULL);
-		XtOverrideTranslations(colorMemory[indx],
-		       XtParseTranslationTable(user_color_translations));
-	    } else {
-		/* already exists, just set its color and map it */
-		FirstArg(XtNforeground, x_bg_color.pixel);
-		NextArg(XtNbackground, (all_colors_available? 
-			user_colors[indx].pixel: x_fg_color.pixel));
-		SetValues(colorMemory[indx]);
-		XtManageChild(colorMemory[indx]);
-	    }
-	}
-	return indx;
-}
-
-/*
-   allocate n colormap entries.  If not enough cells are available, switch 
-   to a private colormap.  If we are already using a private colormap return
-   False.  The pixel numbers allocated are returned in the Pixel array pixels[]
-   The new colormap (if used) is set into the main (tool) window and the color
-   popup panel (if it exists)
-*/
-
-Boolean
-alloc_color(pixels,n)
-Pixel	pixels[];
-int	n;
-{
-    int	i;
-    /* allocate them one at a time in case we can't allocate all of them.  If that
-       is the case then we switch colormaps and allocate the rest */
-    for (i=0; i<n; i++) {
-	if (!XAllocColorCells(tool_d, tool_cm, 0, &plane_masks, 0, &pixels[i], 1)) {
-	    /* try again with new colormap */
-	    if (!switch_colormap() ||
-	        (!XAllocColorCells(tool_d, tool_cm, 0, &plane_masks, 0, &pixels[i], 1))) {
-		    put_msg("Cannot define user colors.");
-		    return False;
-	    }
-        }
-    }
-    return True;
-}
-
-/* switch colormaps to private one and reallocate the colors we already used. */
-/* Return False if already switched or if can't allocate new colormap */
-
-Boolean
-switch_colormap()
-{
-	if (swapped_cmap || appres.dont_switch_cmap) {
-	    return False;
-	}
-	if ((newcmap = XCopyColormapAndFree(tool_d, tool_cm)) == 0) {
-	    file_msg("Cannot allocate new colormap.");
-	    return False;
-	}
-	/* swap colormaps */
-	tool_cm = newcmap;
-	swapped_cmap = True;
-	/* and tell the window manager to install it */
-	if (pen_color_button && pen_color_button->panel &&
-	    XtWindow(pen_color_button->panel) != 0)
-		set_cmap(XtWindow(pen_color_button->panel));
-	if (tool_w)
-	    set_cmap(tool_w);
-	file_msg("Switching to private colormap.");
-	return True;
+	_pick_memory(colorMemory[current_memory]);
+	colorUsed[current_memory]=True;
 }
 
 /* delete a color memory (current_memory) from the user colors */
@@ -987,17 +980,17 @@ XtPointer closure, ptr;
 	int save_mem, save_edit;
 	int i;
 	if (current_memory == -1 || num_usr_cols <= 0) {
-		XBell(tool_d,0);
+		beep();
 		return;
 	}
 	/* only allow deletion of this color of no object in the figure uses it */
 	if (color_used(current_memory+NUM_STD_COLS, &objects)) {
 		put_msg("That color is in use by an object in the figure");
-		XBell(tool_d,0);
+		beep();
 		return;
 	}
 	/* get rid of the box drawn around this cell */
-	draw_a_dotted_box(colorMemory[current_memory], unboxedGC);
+	draw_a_box(colorMemory[current_memory], unboxedGC);
 	/* save it to undelete */
 	undel_user_color = user_colors[current_memory];
 	del_color_cell(current_memory);
@@ -1039,6 +1032,205 @@ XtPointer closure, ptr;
 	colorUsed[indx] = True;
 }
 
+/* lookup color from another window - the user clicks the mouse
+   on a color and a new cell is created with that color */
+
+static void
+lookup_color(w, closure, ptr)
+Widget w;
+XtPointer closure, ptr;
+{
+    Colormap	cmap;
+    Pixel	p;
+    XColor	xcol;
+
+    /* grab the server to get a pixel from a window */
+    DoGrabPixel(w, &p, &cmap);
+    /* make a new cell and set current_memory to that */
+    add_color(w, closure, ptr);
+
+    xcol.pixel = p;
+    xcol.flags = DoRed | DoGreen | DoBlue;
+    XQueryColor(XtDisplay(w), cmap, &xcol);
+
+    /* and store the chosen color */
+    user_colors[current_memory].red = xcol.red;
+    user_colors[current_memory].green = xcol.green;
+    user_colors[current_memory].blue = xcol.blue;
+    user_colors[current_memory].flags = DoRed|DoGreen|DoBlue;
+    set_user_color(current_memory);
+}
+
+/* add a widget to the user color list with color r,g,b */
+/* call with use_exist true if you wish to allocate cell <indx> explicitly */
+/* also increment num_usr_cols if we add a colorcell beyond the current number */
+/* return the colorcell number (0..MAX_USR_COLS-1) */
+/* return -1 if MAX_USR_COLS already used */
+
+int
+add_color_cell(use_exist, indx, r, g, b)
+Boolean	use_exist;
+int	indx;
+int	r,g,b;
+{
+	int	i;
+	char	labl[5];
+	Boolean	new;
+	Pixel	cell_fg;
+
+	if (all_colors_available) {
+	    /* try to get a colorcell */
+	    if (!alloc_color_cells(pixels,1)) {
+		put_msg("Can't allocate user color, not enough colorcells");
+		return 0;
+	    }
+	}
+	if (!use_exist) {
+		/* first look for color cell available in the middle */
+		new = False;
+		for (i=0; i<num_usr_cols; i++)
+		    if (colorFree[i])
+			break;
+		indx = i;
+	}
+
+	/* if a space is free but there was never a color there, must create one */
+	if (indx<num_usr_cols && colorMemory[indx]==0)
+	    new = True;
+
+	/* if not, increment num_usr_cols */
+	if (indx>= num_usr_cols) {
+	    if (num_usr_cols >= MAX_USR_COLS-1)
+		return -1;
+	    if (use_exist)
+		num_usr_cols = indx+1;
+	    else
+		num_usr_cols++;
+	    new = True;
+	}
+
+	user_colors[indx].red = r*256;
+	user_colors[indx].green = g*256;
+	user_colors[indx].blue = b*256;
+	user_colors[indx].flags = DoRed|DoGreen|DoBlue;
+	user_colors[indx].pixel = pixels[0];
+	/* in case we have read-only colormap, get the pixel value now */
+	YStoreColor(tool_cm,&user_colors[indx]);
+	/* and put it in main colors */
+	colors[NUM_STD_COLS+indx] = user_colors[indx].pixel;
+
+	colorUsed[indx] = False;
+	colorFree[indx] = False;
+
+	/* if the color popup has been created create the widgets */
+	if (pen_color_button->panel) {
+	    if (new) {
+		/* put the user color number in all the time */
+		sprintf(labl,"%d", indx+NUM_STD_COLS);
+		colorMemory[indx] = XtVaCreateManagedWidget("colorMemory",
+			labelWidgetClass, userBox,
+			XtNlabel, labl, 
+			XtNforeground, x_bg_color.pixel,
+			XtNbackground, (all_colors_available? 
+				user_colors[indx].pixel: x_fg_color.pixel),
+			XtNwidth, USR_COL_W, XtNheight, USR_COL_H,
+			XtNborderWidth, 1, NULL);
+		XtOverrideTranslations(colorMemory[indx],
+		       XtParseTranslationTable(user_color_translations));
+	    } else {
+		/* already exists, just set its color and map it */
+		FirstArg(XtNforeground, x_bg_color.pixel);
+		NextArg(XtNbackground, (all_colors_available? 
+			user_colors[indx].pixel: x_fg_color.pixel));
+		SetValues(colorMemory[indx]);
+		XtManageChild(colorMemory[indx]);
+	    }
+	}
+	/* now set the color of the widget if TrueColor, etc. */
+	if (all_colors_available)
+	    set_user_color(indx);
+
+	return indx;
+}
+
+/*
+   allocate n colormap entries.  If not enough cells are available, switch 
+   to a private colormap.  If we are already using a private colormap return
+   False.  The pixel numbers allocated are returned in the Pixel array pixels[]
+   The new colormap (if used) is set into the main (tool) window and the color
+   popup panel (if it exists)
+*/
+
+Boolean
+alloc_color_cells(pixels,n)
+Pixel	pixels[];
+int	n;
+{
+    int	i;
+
+    /* allocate them one at a time in case we can't allocate all of them.
+       If that is the case then we switch colormaps and allocate the rest */
+    for (i=0; i<n; i++) {
+      switch( tool_vclass ){
+/*
+ * Use XAllocColorCells "to allocate read/write color cells and color plane
+ * combinations for GrayScale and PseudoColor models"
+ */
+	case GrayScale:
+	case PseudoColor:
+	    if (!XAllocColorCells(tool_d, tool_cm, 0, &plane_masks, 0, &pixels[i], 1)) {
+	        /* try again with new colormap */
+		if (!switch_colormap() ||
+	            (!XAllocColorCells(tool_d, tool_cm, 0, &plane_masks, 0, &pixels[i], 1))) {
+		    put_msg("Cannot define user colors.");
+		    return False;
+		}
+	    }
+	    break;
+/*
+ * Do not attempt to allocate color resources for static visuals
+ * (but assume the colors can be achieved).
+ */
+	case StaticGray:
+	case StaticColor:
+	case DirectColor:
+	case TrueColor:
+	    break;
+
+        default:
+	    put_msg("Cannot define user colors in the Unknown Visual.");
+	    return False;
+      }
+    }
+    return True;
+}
+
+/* switch colormaps to private one and reallocate the colors we already used. */
+/* Return False if already switched or if can't allocate new colormap */
+
+Boolean
+switch_colormap()
+{
+	if (swapped_cmap || appres.dont_switch_cmap) {
+	    return False;
+	}
+	if ((newcmap = XCopyColormapAndFree(tool_d, tool_cm)) == 0) {
+	    file_msg("Cannot allocate new colormap.");
+	    return False;
+	}
+	/* swap colormaps */
+	tool_cm = newcmap;
+	swapped_cmap = True;
+	/* and tell the window manager to install it */
+	if (pen_color_button && pen_color_button->panel &&
+	    XtWindow(pen_color_button->panel) != 0)
+		set_cmap(XtWindow(pen_color_button->panel));
+	if (tool_w)
+	    set_cmap(tool_w);
+	file_msg("Switching to private colormap.");
+	return True;
+}
+
 
 /* delete a color memory cell (indx) from the widgets and arrays */
 
@@ -1148,18 +1340,28 @@ Widget		 w;
 char		*dum;
 XButtonEvent	*ev;
 {
+	int	i,indx;
+
+	/* put the color pixel value in the table */
+	/* this is done here because if the visual is TrueColor, etc. the value of
+	   the pixel changes with the color itself */
+	for (i=0; i<=1; i++) {
+	    indx = mixed_color_indx[i];
+	    if (indx >= NUM_STD_COLS)
+		colors[indx] = user_colors[indx-NUM_STD_COLS].pixel;
+	}
+
 	/* have either the fill or pen color been modified? */
 
 	if (modified[0]) {
 	    cur_pencolor = mixed_color_indx[0];
-	    show_pen_color();
+	    show_pen_color(); /* update the button in the indicator panel */
 	}
 	if (modified[1]) {
 	    cur_fillcolor = mixed_color_indx[1];
-	    show_fill_color();
+	    show_fill_color(); /* update the button in the indicator panel */
 	}
 	modified[0] = modified[1] = False;
-	/* update the button in the indicator panel */
 	choice_panel_dismiss();
 }
 
@@ -1215,7 +1417,8 @@ XButtonEvent	*ev;
 	rgb_values[edit_fill].g = mixed_color[edit_fill].green;
 	rgb_values[edit_fill].b = mixed_color[edit_fill].blue;
 	if (all_colors_available)
-	    XStoreColor(tool_d, tool_cm, &mixed_color[edit_fill]);
+	    set_mixed_color(edit_fill);
+		
 	/* undraw any box around the current user-memory cell */
 	if (current_memory != -1)
 	    erase_boxed((Widget)NULL, (XEvent *)NULL,
@@ -1244,6 +1447,12 @@ Widget w;
 XEvent *event;
 String *params;
 Cardinal *num_params;
+{
+	_pick_memory(w);
+}
+
+_pick_memory(w)
+Widget w;
 {
 	int	i;
 
@@ -1255,7 +1464,7 @@ Cardinal *num_params;
 	    if (w == colorMemory[i]) {
 		/* erase box around old memory cell */
 		if (current_memory >= 0)
-		    draw_a_dotted_box(colorMemory[current_memory], unboxedGC);
+		    draw_a_box(colorMemory[current_memory], unboxedGC);
 		/* new memory cell */
 		current_memory = i;
 		draw_boxed((Widget)NULL, (XEvent *)NULL,
@@ -1273,7 +1482,7 @@ Cardinal *num_params;
 			CHANGE_BLUE(current_memory);
 			do_change = True;
 			if (all_colors_available)
-			    XStoreColor(tool_d, tool_cm, &mixed_color[edit_fill]);
+			    set_mixed_color(edit_fill);
 			update_scrl_triple((Widget)NULL, (XEvent *)NULL,
 				(String *)NULL, (Cardinal *)NULL);
 		} else {
@@ -1284,7 +1493,7 @@ Cardinal *num_params;
 			user_colors[current_memory].blue =
 					mixed_color[edit_fill].blue;
 			if (all_colors_available)
-			    XStoreColor(tool_d, tool_cm, &user_colors[current_memory]);
+			    set_user_color(current_memory);
 		}
 		/* activate the delete color button */
 		XtSetSensitive(delColor, True);
@@ -1345,8 +1554,9 @@ Cardinal *num_params;
 }
 
 
-/* draw a dashed box around the given widget */
-draw_a_dotted_box(w, gc)
+/* draw or erase a dashed or solid box around the given widget */
+
+draw_a_box(w, gc)
 Widget w;
 GC gc;
 {
@@ -1368,7 +1578,6 @@ GC gc;
 	XDrawRectangle(tool_d, XtWindow(userBox), gc, x, y, width, height);
 }
 
-
 static void
 expose_draw_boxed(w, event, params, num_params)
 Widget w;
@@ -1389,7 +1598,7 @@ Cardinal *num_params;
 {
 	if (current_memory < 0)
 		return;
-	draw_a_dotted_box(colorMemory[current_memory],
+	draw_a_box(colorMemory[current_memory],
 			colorUsed[current_memory] ? boxedChangedGC : boxedGC);
 }
 
@@ -1403,7 +1612,7 @@ Cardinal *num_params;
 {
 	if (current_memory < 0)
 		return;
-	draw_a_dotted_box(colorMemory[current_memory], unboxedGC);
+	draw_a_box(colorMemory[current_memory], unboxedGC);
 }
 
 static void
@@ -1502,7 +1711,7 @@ Cardinal *num_params;
 	}
 	if ((strlen(hexvalue) != 7) ||
 	   (sscanf(hexvalue,"#%02x%02x%02x",&red,&green,&blue) != 3)) {
-		XBell(tool_d,0);
+		beep();
 		put_msg("Bad hex value");
 		return;
 	}
@@ -1673,12 +1882,13 @@ Cardinal *num_params;
 StoreMix_and_Mem()
 {
 	if (all_colors_available)
-	    XStoreColor(tool_d, tool_cm, &mixed_color[edit_fill]);
+	    set_mixed_color(edit_fill);
+	set_mixed_color(edit_fill);
 	user_colors[current_memory].red = mixed_color[edit_fill].red;
 	user_colors[current_memory].green = mixed_color[edit_fill].green;
 	user_colors[current_memory].blue = mixed_color[edit_fill].blue;
 	if (all_colors_available)
-	    XStoreColor(tool_d, tool_cm, &user_colors[current_memory]);
+	    set_user_color(current_memory);
 }
 
 static void
@@ -1689,6 +1899,7 @@ XtPointer closure, change;
 	Boolean going_up = (int) change < 0;
 	int which = (int) closure;
 	int pos = 0;
+	float blip = 1.0/256.0;
 
 	switch (which) {
 		case S_RED:
@@ -1704,9 +1915,16 @@ XtPointer closure, change;
 			pos = 255 - (int)(locked_top * 255 + 0.5);
 			break;
 		case S_HUE:
+			hsv_values.h += (going_up? -blip: blip);
+			ThumbHSV(w, hsv_values.h);
+			return;
 		case S_SAT:
+			hsv_values.s += (going_up? -blip: blip);
+			ThumbHSV(w, hsv_values.s);
+			return;
 		case S_VAL:
-			/* Not yet implemented */
+			hsv_values.v += (going_up? -blip: blip);
+			ThumbHSV(w, hsv_values.v);
 			return;
 		default:
 			fprintf(stderr, "Oops Scroll calldata invalid\n");
@@ -1748,15 +1966,23 @@ XtPointer closure, ptr;
 			break;
 	}
 
-	rgb_values[edit_fill] = HSVToRGB(hsv_values);
+	ThumbHSV(w, top);
+}
 
+ThumbHSV(w, top)
+Widget w;
+double top;
+{
+	rgb_values[edit_fill] = HSVToRGB(hsv_values);
 	XawScrollbarSetThumb(w, top, (float)THUMB_H);
 
+	/* don't update the scrollbars yet */
 	do_change = False;
 	pass_value = 1.0 - rgb_values[edit_fill].r/65536.0;
 	Thumbed(redScroll, (XtPointer)S_RED, (XtPointer)(&pass_value));
 	pass_value = 1.0 - rgb_values[edit_fill].g/65536.0;
 	Thumbed(greenScroll, (XtPointer)S_GREEN, (XtPointer)(&pass_value));
+	/* now update the scrollbars */
 	do_change = True;
 	pass_value = 1.0 - rgb_values[edit_fill].b/65536.0;
 	Thumbed(blueScroll, (XtPointer)S_BLUE, (XtPointer)(&pass_value));
@@ -1879,19 +2105,19 @@ show_pen_color()
 	sprintf(colorname,"%d",cur_pencolor);
     }
     /* first erase old colorname */
-    XDrawImageString(tool_d, pen_color_button->normalPM, ind_button_gc, 3, 25,
+    XDrawImageString(tool_d, pen_color_button->pixmap, ind_button_gc, 3, 25,
 	      "        ", 8);
     /* now fill the color rectangle with the new color */
     XSetForeground(tool_d, pen_color_gc, color);
-    XFillRectangle(tool_d, pen_color_button->normalPM, pen_color_gc,
+    XFillRectangle(tool_d, pen_color_button->pixmap, pen_color_gc,
 			pen_color_button->sw_width - 30, 4, 26, 26);
     /* and put the color name in the button also */
-    XDrawImageString(tool_d, pen_color_button->normalPM, ind_button_gc, 3, 25,
+    XDrawImageString(tool_d, pen_color_button->pixmap, ind_button_gc, 3, 25,
 	      colorname, strlen(colorname));
     FirstArg(XtNbackgroundPixmap, 0);
     SetValues(pen_color_button->button);
     /* put the pixmap in the widget background */
-    FirstArg(XtNbackgroundPixmap, pen_color_button->normalPM);
+    FirstArg(XtNbackgroundPixmap, pen_color_button->pixmap);
     SetValues(pen_color_button->button);
 }
 
@@ -1946,19 +2172,19 @@ show_fill_color()
 	sprintf(colorname,"%d",cur_fillcolor);
     }
     /* first erase old colorname */
-    XDrawImageString(tool_d, fill_color_button->normalPM, ind_button_gc, 3, 25,
+    XDrawImageString(tool_d, fill_color_button->pixmap, ind_button_gc, 3, 25,
 	      "        ", 8);
     /* now fill the color rectangle  with the new fill color */
     XSetForeground(tool_d, fill_color_gc, color);
-    XFillRectangle(tool_d, fill_color_button->normalPM, fill_color_gc,
+    XFillRectangle(tool_d, fill_color_button->pixmap, fill_color_gc,
 			fill_color_button->sw_width - 30, 4, 26, 26);
     /* and put the color name in the button also */
-    XDrawImageString(tool_d, fill_color_button->normalPM, ind_button_gc, 3, 25,
+    XDrawImageString(tool_d, fill_color_button->pixmap, ind_button_gc, 3, 25,
 	      colorname, strlen(colorname));
     FirstArg(XtNbackgroundPixmap, 0);
     SetValues(fill_color_button->button);
     /* put the pixmap in the widget background */
-    FirstArg(XtNbackgroundPixmap, fill_color_button->normalPM);
+    FirstArg(XtNbackgroundPixmap, fill_color_button->pixmap);
     SetValues(fill_color_button->button);
 }
 
@@ -1986,7 +2212,10 @@ Window window;
  */
 
 /*
- * $Log:	color.c,v $
+ * $Log: w_color.c,v $
+ * Revision 1.1  1995/02/28  15:40:16  feuille
+ * Initial revision
+ *
  * Revision 1.2  90/06/30  14:32:48  rlh2
  * patchlevel 1
  * 
@@ -2120,3 +2349,186 @@ float	rr, gg, bb;
 	rgb.b = (int)(0.5 + bb * MAX_INTENSITY);
 	return rgb;
 }
+
+/* +-------------------------------------------------------------------+ */
+/* | Copyright 1993, David Koblas (koblas@netcom.com)                  | */
+/* | Copyright 1995, 1996 Torsten Martinsen (bullestock@dk-online.dk)  | */
+/* |                                                                   | */
+/* | Permission to use, copy, modify, and to distribute this software  | */
+/* | and its documentation for any purpose is hereby granted without   | */
+/* | fee, provided that the above copyright notice appear in all       | */
+/* | copies and that both that copyright notice and this permission    | */
+/* | notice appear in supporting documentation.  There is no           | */
+/* | representations about the suitability of this software for        | */
+/* | any purpose.  this software is provided "as is" without express   | */
+/* | or implied warranty.                                              | */
+/* |                                                                   | */
+/* +-------------------------------------------------------------------+ */
+
+
+/*
+**  Grab the pixel value from some other window
+**
+**  Store pixel value in *p and colormap ID in *cmap unless they are NULL.
+**
+**   General strategy:
+**     Grab the cursor
+**     Wait for the up/down button event
+**     Lookup what window the event is over
+**     Query the pixel value
+ */
+static void 
+DoGrabPixel(w, p, cmap)
+    Widget	 w;
+    Pixel	*p;
+    Colormap	*cmap;
+{
+    int x, y, nx, ny;
+    XImage *xim;
+    Colormap amap;
+    Window root = RootWindowOfScreen(XtScreen(w));
+    Window window;
+    Display *dpy = XtDisplay(w);
+
+    doGrab(w, 0, 0, &x, &y);
+
+    if (cmap == NULL)
+	cmap = &amap;
+
+    xyToWindowCmap(dpy, x, y, root, &nx, &ny, &window, cmap);
+
+    xim = XGetImage(dpy, window, nx, ny, 1, 1, AllPlanes, ZPixmap);
+
+    if (p != NULL)
+	*p = XGetPixel(xim, 0, 0);
+
+    XDestroyImage(xim);
+}
+
+/*
+ * Grab a rectangle of some window.
+ * Zero width and height specifies to just grab a single pixel.
+ * Returns coords of event.
+ */
+
+/*
+**  Convenience function for doing server pointer grabs
+ */
+#define GRAB_INTERVAL	30
+
+typedef struct {
+    XtAppContext app;
+    Display *dpy;
+    GC gc;
+    Window root;
+    Boolean drawn;
+    int x, y, ox, oy, width, height;
+    XtIntervalId id;
+} GrabInfo;
+
+static void 
+doGrab(w, width, height, x, y)
+    Widget	 w;
+    int		 width, height;
+    int		*x, *y;
+{
+    Display *dpy = XtDisplay(w);
+    XtAppContext app = XtWidgetToApplicationContext(w);
+    Window root = DefaultRootWindow(dpy);
+    XEvent event;
+    Cursor cursor = XCreateFontCursor(dpy, XC_crosshair);
+    int count = 0;
+    GrabInfo *info = NULL;
+
+    /* Set up grab cursor */
+    if (XGrabPointer(dpy, root, False,
+		     info == NULL ? ButtonPressMask | ButtonReleaseMask
+	       : ButtonPressMask | ButtonReleaseMask | PointerMotionMask,
+		     GrabModeSync, GrabModeAsync,
+		     root, cursor, CurrentTime))
+	return;
+
+    do {
+	XAllowEvents(dpy, SyncPointer, CurrentTime);
+	XtAppNextEvent(app, &event);
+	if (event.type == ButtonPress)
+	    count++;
+	else if (event.type == ButtonRelease) {
+	    if (count == 1)
+		break;
+	    else
+		count--;
+	} else if (event.type == MotionNotify) {
+	    info->x = event.xmotion.x;
+	    info->y = event.xmotion.y;
+	} else
+	    XtDispatchEvent(&event);
+    }
+    while (True);
+
+    XUngrabPointer(dpy, CurrentTime);
+
+    if (info != NULL) {
+	/* remove grab cursor */
+	if (info->drawn)
+	    XDrawRectangle(info->dpy, info->root, info->gc,
+			   info->ox - info->width / 2,
+			   info->oy - info->height / 2,
+			   info->width, info->height);
+	XtRemoveTimeOut(info->id);
+	XFreeGC(XtDisplay(w), info->gc);
+	XtFree((XtPointer) info);
+    }
+    *x = event.xbutton.x;
+    *y = event.xbutton.y;
+}
+
+/*
+ * Given coords x,y in the 'base' window, descend the window hierarchy
+ * and find the child window of class InputOutput containing those
+ * coordinates. Return coords in child window in (*nx,*ny).
+ * If the child window has a colormap, return that; otherwise return
+ * the default colormap for the display.
+ */
+static void 
+xyToWindowCmap(dpy, x, y, base, nx, ny, window, cmap)
+    Display	*dpy;
+    int		 x, y;
+    Window	 base;
+    int		*nx, *ny;
+    Window	*window;
+    Colormap	*cmap;
+{
+    Window twin;
+    Colormap tmap;
+    Window child, sub;
+    XWindowAttributes attr;
+
+    twin = base;
+    tmap = None;
+
+    sub = base;
+    *nx = x;
+    *ny = y;
+
+    while (sub != None) {
+	x = *nx;
+	y = *ny;
+	child = sub;
+	XTranslateCoordinates(dpy, base, child, x, y, nx, ny, &sub);
+	base = child;
+
+	XGetWindowAttributes(dpy, child, &attr);
+	if (attr.class == InputOutput && attr.colormap != None) {
+	    tmap = attr.colormap;
+	    twin = child;
+	}
+    }
+
+    if (tmap == None)
+	*cmap = DefaultColormap(dpy, DefaultScreen(dpy));
+    else
+	*cmap = tmap;
+    *window = twin;
+}
+
