@@ -1,3 +1,5 @@
+#ifndef W_COLOR_H
+#define W_COLOR_H
 /*
  * FIG : Facility for Interactive Generation of figures
  * Copyright (c) 1987 Christopher A. Kent
@@ -8,11 +10,14 @@
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
  * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.  This license includes without
- * limitation a license to do the foregoing actions under any patents of
- * the party supplying this software to the X Consortium.
+ * and/or sell copies of the Software subject to the restriction stated
+ * below, and to permit persons who receive copies from any such party to
+ * do so, with the only requirement being that this copyright notice remain
+ * intact.
+ * This license includes without limitation a license to do the foregoing
+ * actions under any patents of the party supplying this software to the 
+ * X Consortium.
+ *
  */
 
 /* 
@@ -24,8 +29,14 @@
  * Date:	Sun Dec 13 1987
  */
 
+Boolean switch_colormap();
+Boolean alloc_color_cells();
+
 /*
- * $Log:	color.h,v $
+ * $Log: w_color.h,v $
+ * Revision 1.1  1995/02/28  15:40:16  feuille
+ * Initial revision
+ *
  * Revision 1.2  90/06/30  14:33:12  rlh2
  * patchlevel 1
  * 
@@ -63,3 +74,4 @@ RGB	PctToRGB();
 HSV	PctToHSV();
 RGB	CMYToRGB();
 CMY	RGBToCMY();
+#endif /* W_COLOR_H */
