@@ -1,3 +1,5 @@
+#ifndef W_INDPANEL_H
+#define W_INDPANEL_H
 /*
  * FIG : Facility for Interactive Generation of figures
  * Copyright (c) 1991 by Paul King
@@ -8,11 +10,14 @@
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
  * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.  This license includes without
- * limitation a license to do the foregoing actions under any patents of
- * the party supplying this software to the X Consortium.
+ * and/or sell copies of the Software subject to the restriction stated
+ * below, and to permit persons who receive copies from any such party to
+ * do so, with the only requirement being that this copyright notice remain
+ * intact.
+ * This license includes without limitation a license to do the foregoing
+ * actions under any patents of the party supplying this software to the 
+ * X Consortium.
+ *
  */
 
 #include "w_icons.h"
@@ -99,13 +104,13 @@ extern	Dimension UPD_CTRL_WD;		/* actual width is det. in setup_ind_panel */
 typedef struct choice_struct {
     int		    value;
     icon_struct	   *icon;
-    Pixmap	    normalPM,blackPM;
+    Pixmap	    pixmap;
 }		choice_info;
 
 typedef struct ind_sw_struct {
     int		    type;	/* one of I_CHOICE .. I_FVAL */
     int		    func;
-    char	    line1[14], line2[6];
+    char	    line1[16], line2[8];
     int		    sw_width;
     int		   *i_varadr;
     float	   *f_varadr;
@@ -119,7 +124,7 @@ typedef struct ind_sw_struct {
     Widget	    button;
     Widget	    formw;
     Widget	    updbut;
-    Pixmap	    normalPM;
+    Pixmap	    pixmap;
     Widget	    panel;	/* to keep track if already created */
 }		ind_sw_info;
 
@@ -128,3 +133,8 @@ extern ind_sw_info ind_switches[];
 
 extern unsigned char	arrow0_bits[], arrow1_bits[], arrow2_bits[], arrow3_bits[];
 extern unsigned char	arrow4_bits[], arrow5_bits[], arrow6_bits[];
+
+/* pointer to the color popup so it can be re-used in the popup edit panel */
+extern ind_sw_info *color_popup;
+
+#endif /* W_INDPANEL_H */
