@@ -1,3 +1,5 @@
+#ifndef W_MOUSEFUN_H
+#define W_MOUSEFUN_H
 /*
  * FIG : Facility for Interactive Generation of figures
  * Copyright (c) 1991 by Paul King
@@ -8,11 +10,14 @@
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
  * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.  This license includes without
- * limitation a license to do the foregoing actions under any patents of
- * the party supplying this software to the X Consortium.
+ * and/or sell copies of the Software subject to the restriction stated
+ * below, and to permit persons who receive copies from any such party to
+ * do so, with the only requirement being that this copyright notice remain
+ * intact.
+ * This license includes without limitation a license to do the foregoing
+ * actions under any patents of the party supplying this software to the 
+ * X Consortium.
+ *
  */
 
 void		init_mousefun();
@@ -37,3 +42,4 @@ void		draw_mousefun_kbd();
 void		clear_mousefun_kbd();
 void		init_mousefun_actions();
 extern String	kbd_translations;
+#endif /* W_MOUSEFUN_H */
