@@ -9,35 +9,40 @@
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
  * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.  This license includes without
- * limitation a license to do the foregoing actions under any patents of
- * the party supplying this software to the X Consortium.
+ * and/or sell copies of the Software subject to the restriction stated
+ * below, and to permit persons who receive copies from any such party to
+ * do so, with the only requirement being that this copyright notice remain
+ * intact.
+ * This license includes without limitation a license to do the foregoing
+ * actions under any patents of the party supplying this software to the 
+ * X Consortium.
+ *
  */
 
 #include "fig.h"
 #include "figx.h"
-#include "mode.h"
 #include "resources.h"
+#include "object.h"
+#include "mode.h"
+#include "w_export.h"
+#include "w_print.h"
 #include "w_icons.h"
 #include "w_setup.h"
 #include "w_util.h"
 
-extern Widget	make_popup_menu();
+/* EXPORTS */
 extern char    *panel_get_value();
 extern char     batch_file[];
 extern Boolean  batch_exists;
 extern char    *shell_protect_string();
 
-/* from w_export.c */
-extern Widget		export_orient_panel;
-extern Widget		export_just_panel;
-
-/* global so w_cmdpanel.c and w_export.c can access it */
+Widget		print_popup;	/* the main export popup */
 Widget		print_orient_panel;
-/* global so f_read.c and w_export.c can access it */
 Widget		print_just_panel;
+Widget		print_papersize_panel;
+Widget		print_multiple_panel;
+Widget		print_mag_text;
+void		print_update_figure_size();
 
 /* LOCAL */
 
@@ -47,28 +52,65 @@ static Widget	orient_menu, orient_lab;
 static void	just_select();
 static Widget	just_menu, just_lab;
 
-static Widget	print_panel, print_popup, dismiss, print, 
+static void	papersize_select();
+static Widget	papersize_menu, papersize_lab;
+
+static void	multiple_select();
+static Widget	multiple_menu, multiple_lab;
+
+static Widget	print_panel, dismiss, print, 
 		printer_text, param_text, printer_lab, param_lab, clear_batch, print_batch, 
-		mag_lab, print_w, mag_text, num_batch_lab, num_batch;
+		print_w, num_batch_lab, num_batch;
+
+static Widget	mag_lab;
+static Widget	size_lab;
+
+static void	update_figure_size();
+static Widget	fitpage;
+static void	fit_page();
+
 static Position xposn, yposn;
-static String   prin_translations =
+static String   prn_translations =
         "<Message>WM_PROTOCOLS: DismissPrin()\n";
 static void     print_panel_dismiss(), do_clear_batch();
+static void	get_magnif();
+static XtCallbackProc update_mag();
 void		do_print(), do_print_batch();
-static XtActionsRec     prin_actions[] =
+
+String  print_translations =
+        "<Key>Return: UpdateMag()\n\
+	Ctrl<Key>J: UpdateMag()\n\
+	Ctrl<Key>M: UpdateMag()\n\
+	Ctrl<Key>X: EmptyTextKey()\n\
+	Ctrl<Key>U: multiply(4)\n\
+	<Key>F18: PastePanelKey()\n";
+
+static XtActionsRec     prn_actions[] =
 {
     {"DismissPrin", (XtActionProc) print_panel_dismiss},
     {"dismiss", (XtActionProc) print_panel_dismiss},
     {"print_batch", (XtActionProc) do_print_batch},
     {"clear_batch", (XtActionProc) do_clear_batch},
     {"print", (XtActionProc) do_print},
+    {"UpdateMag", (XtActionProc) update_mag},
 };
 
+/* callback list to keep track of magnification window */
+
+static XtCallbackRec mag_callback[] = {
+	{(XtCallbackProc)update_mag, (XtPointer)NULL},
+	{(XtCallbackProc)NULL, (XtPointer)NULL},
+	};
+
 static void
 print_panel_dismiss(w, ev)
     Widget	    w;
     XButtonEvent   *ev;
 {
+    /* first get magnification in case it changed */
+    /* the other things like paper size, justification, etc. are already
+       updated because they are from menus */
+    get_magnif();
     XtPopdown(print_popup);
     XtSetSensitive(print_w, True);
 }
@@ -79,11 +121,11 @@ void
 do_print(w)
     Widget	    w;
 {
-	DeclareArgs(1);
-	float	    mag;
+	DeclareArgs(2);
 	char	   *printer_val;
 	char	   *param_val;
-	char	    cmd[100];
+	char	    cmd[255],cmd2[255];
+	char	   *c1, *c2;
 
 	if (emptyfigure_msg(print_msg) && !batch_exists)
 		return;
@@ -92,64 +134,129 @@ do_print(w)
 	   resources necessary (e.g. printer name etc.) */
 	if (!print_popup) 
 		create_print_panel(w);
-	mag = (float) atof(panel_get_value(mag_text)) / 100.0;
-	if (mag <= 0.0)
-	    mag = 1.0;
+
+	/* get the magnification into appres.magnification */
+	get_magnif();
+
+	/* update the figure size (magnification * bounding_box) */
+	print_update_figure_size();
 
 	FirstArg(XtNstring, &printer_val);
 	GetValues(printer_text);
 	FirstArg(XtNstring, &param_val);
 	GetValues(param_text);
-	if (batch_exists)
-	    {
+	if (batch_exists) {
 	    gen_print_cmd(cmd,batch_file,printer_val,param_val);
 	    if (system(cmd) != 0)
 		file_msg("Error during PRINT");
 	    /* clear the batch file and the count */
 	    do_clear_batch(w);
+	} else {
+	    strcpy(cmd, param_val);
+	    /* see if the user wants the filename in the param list (%f) */
+	    if (!strstr(cur_filename,"%f")) {	/* don't substitute if the filename has a %f */
+		while (c1=strstr(cmd,"%f")) {
+		    strcpy(cmd2, c1+2);		/* save tail */
+		    strcpy(c1, cur_filename);	/* change %f to filename */
+		    strcat(c1, cmd2);		/* append tail */
+		}
 	    }
-	else
-	    {
-	    print_to_printer(printer_val, mag, appres.flushleft, param_val);
-	    }
+	    print_to_printer(printer_val, appres.magnification, cmd);
+	}
 }
 
-gen_print_cmd(cmd,file,printer,pr_params)
-    char	   *cmd;
-    char	   *file;
-    char	   *printer;
-    char	   *pr_params;
+/* calculate the magnification needed to fit the figure to the page size */
+
+static void
+fit_page()
 {
-    if (emptyname(printer)) {	/* send to default printer */
-#if (defined(SYSV) || defined(SVR4)) && !defined(BSDLPR)
-	sprintf(cmd, "lp %s %s", 
-		pr_params,
-		shell_protect_string(file));
-#else
-	sprintf(cmd, "lpr %s -J %s %s",
-		pr_params,
-		shell_protect_string(file),
-		shell_protect_string(file));
-#endif
-	put_msg("Printing figure on default printer in %s mode ...     ",
-		appres.landscape ? "LANDSCAPE" : "PORTRAIT");
-    } else {
-#if (defined(SYSV) || defined(SVR4)) && !defined(BSDLPR)
-	sprintf(cmd, "lp %s, -d%s %s",
-		pr_params,
-		printer, 
-		shell_protect_string(file));
-#else
-	sprintf(cmd, "lpr %s -J %s -P%s %s", 
-		pr_params,
-		shell_protect_string(file),
-		printer,
-		shell_protect_string(file));
-#endif
-	put_msg("Printing figure on printer %s in %s mode ...     ",
-		printer, appres.landscape ? "LANDSCAPE" : "PORTRAIT");
+	int	lx,ly,ux,uy;
+	float	wd,ht,pwd,pht;
+	char	buf[60];
+	DeclareArgs(2);
+
+	/* get current size of figure */
+	compound_bound(&objects, &lx, &ly, &ux, &uy);
+	wd = ux-lx;
+	ht = uy-ly;
+
+	/* if there is no figure, return now */
+	if (wd == 0 || ht == 0)
+	    return;
+
+	/* get paper size minus a half inch margin all around */
+	pwd = paper_sizes[appres.papersize].width - PIX_PER_INCH;
+	pht = paper_sizes[appres.papersize].height - PIX_PER_INCH;
+	/* swap height and width if landscape */
+	if (appres.landscape) {
+	    ux = pwd;
+	    pwd = pht;
+	    pht = ux;
+	}
+	/* make magnification lesser of ratio of:
+	   page height / figure height or 
+	   page width/figure width
+	*/
+	if (pwd/wd < pht/ht)
+	    appres.magnification = 100.0*pwd/wd;
+	else
+	    appres.magnification = 100.0*pht/ht;
+
+	/* update the magnification widget */
+	sprintf(buf,"%.2f",appres.magnification);
+	FirstArg(XtNstring, buf);
+	SetValues(print_mag_text);
+
+	/* and figure size */
+	update_figure_size();
+}
+
+/* get the magnification from the widget and make it reasonable if not */
+
+static void
+get_magnif()
+{
+	char buf[60];
+	DeclareArgs(2);
+
+	appres.magnification = (float) atof(panel_get_value(print_mag_text));
+	if (appres.magnification <= 0.0)
+	    appres.magnification = 100.0;
+	/* write it back to the widget in case it had a bad value */
+	sprintf(buf,"%.2f",appres.magnification);
+	FirstArg(XtNstring, buf);
+	SetValues(print_mag_text);
+}
+
+/* as the user types in a magnification, update the figure size */
+
+static XtCallbackProc
+update_mag(widget, item, event)
+    Widget	    widget;
+    XtPointer	   *item;
+    XtPointer	   *event;
+{
+    char	   *buf;
+
+    buf = panel_get_value(print_mag_text);
+    appres.magnification = (float) atof(buf);
+    update_figure_size();
+}
+
+static void
+update_figure_size()
+{
+    char buf[60];
+    DeclareArgs(2);
+
+    print_update_figure_size();
+    /* update the export panel's indicators too */
+    if (export_popup) {
+	export_update_figure_size();
+	sprintf(buf,"%.2f",appres.magnification);
+	FirstArg(XtNstring, buf);
+	SetValues(export_mag_text);
     }
-    app_flush();		/* make sure message gets displayed */
 }
 
 static int num_batch_figures=0;
@@ -159,7 +266,6 @@ void
 do_print_batch(w)
     Widget	    w;
 {
-	float	    mag;
 	FILE	   *infp,*outfp;
 	char	    tmp_exp_file[32];
 	char	    str[255];
@@ -178,11 +284,14 @@ do_print_batch(w)
 	batch_exists = True;
 	if (!print_popup) 
 		create_print_panel(w);
-	mag = (float) atof(panel_get_value(mag_text)) / 100.0;
-	if (mag <= 0.0)
-	    mag = 1.0;
 
-	print_to_file(tmp_exp_file, "ps", mag, appres.flushleft, 0, 0);
+	/* get magnification into appres.magnification */
+	get_magnif();
+
+	/* update the figure size (magnification * bounding_box) */
+	print_update_figure_size();
+
+	print_to_file(tmp_exp_file, "ps", appres.magnification, 0, 0);
 	put_msg("Appending to batch file \"%s\" (%s mode) ... done",
 		    batch_file, appres.landscape ? "LANDSCAPE" : "PORTRAIT");
 	app_flush();		/* make sure message gets displayed */
@@ -224,8 +333,8 @@ do_clear_batch(w)
 
 update_batch_count()
 {
-	char	    num[4];
-	DeclareArgs(1);
+	char	    num[10];
+	DeclareArgs(2);
 
 	sprintf(num,"%3d",num_batch_figures);
 	FirstArg(XtNlabel,num);
@@ -246,14 +355,12 @@ orient_select(w, new_orient, garbage)
     Widget	    w;
     XtPointer	    new_orient, garbage;
 {
-    DeclareArgs(1);
-
-    FirstArg(XtNlabel, XtName(w));
-    SetValues(print_orient_panel);
-    /* set export panel too if it exists */
-    if (export_orient_panel)
-	SetValues(export_orient_panel);
-    appres.landscape = (int) new_orient;
+    if (appres.landscape != (int) new_orient) {
+	change_orient();
+	appres.landscape = (int) new_orient;
+	/* make sure that paper size is appropriate */
+	papersize_select(print_papersize_panel, (XtPointer) appres.papersize, (XtPointer) 0);
+    }
 }
 
 static void
@@ -261,7 +368,7 @@ just_select(w, new_just, garbage)
     Widget	    w;
     XtPointer	    new_just, garbage;
 {
-    DeclareArgs(1);
+    DeclareArgs(2);
 
     FirstArg(XtNlabel, XtName(w));
     SetValues(print_just_panel);
@@ -271,15 +378,97 @@ just_select(w, new_just, garbage)
     appres.flushleft = (new_just? True: False);
 }
 
+static void
+papersize_select(w, new_papersize, garbage)
+    Widget	    w;
+    XtPointer	    new_papersize, garbage;
+{
+    DeclareArgs(2);
+    int papersize = (int) new_papersize;
+
+    FirstArg(XtNlabel, paper_sizes[papersize].fname);
+    SetValues(print_papersize_panel);
+    /* change export papersize if it exists */
+    if (export_papersize_panel)
+	SetValues(export_papersize_panel);
+    appres.papersize = papersize;
+}
+
+static void
+multiple_select(w, new_multiple, garbage)
+    Widget	    w;
+    XtPointer	    new_multiple, garbage;
+{
+    DeclareArgs(2);
+    int multiple = (int) new_multiple;
+
+    FirstArg(XtNlabel, multiple_pages[multiple]);
+    SetValues(print_multiple_panel);
+    /* change export multiple if it exists */
+    if (export_multiple_panel)
+	SetValues(export_multiple_panel);
+    appres.multiple = (multiple? True : False);
+    /* if multiple pages, disable justification (must be flush left) */
+    if (appres.multiple) {
+	XtSetSensitive(just_lab, False);
+	XtSetSensitive(print_just_panel, False);
+	if (export_just_panel) {
+	    XtSetSensitive(just_lab, False);
+	    XtSetSensitive(export_just_panel, False);
+	}
+    } else {
+	XtSetSensitive(just_lab, True);
+	XtSetSensitive(print_just_panel, True);
+	if (export_just_panel) {
+	    XtSetSensitive(just_lab, True);
+	    XtSetSensitive(export_just_panel, True);
+	}
+    }
+}
+
+/* update the figure size window */
+
+void
+print_update_figure_size()
+{
+	float	mult;
+	char	*unit;
+	char	buf[40];
+	int	ux,uy,lx,ly;
+	DeclareArgs(2);
+
+	if (!print_popup)
+	    return;
+	mult = appres.INCHES? PIX_PER_INCH : PIX_PER_CM;
+	unit = appres.INCHES? "in": "cm";
+	compound_bound(&objects, &lx, &ly, &ux, &uy);
+	sprintf(buf, "Fig Size: %.1f%s x %.1f%s",
+		(float)(ux-lx)/mult*appres.magnification/100.0,unit,
+		(float)(uy-ly)/mult*appres.magnification/100.0,unit);
+	FirstArg(XtNlabel, buf);
+	SetValues(size_lab);
+}
+
 popup_print_panel(w)
     Widget	    w;
 {
     extern	    Atom wm_delete_window;
+    char	    buf[30];
+    DeclareArgs(2);
 
     set_temp_cursor(wait_cursor);
     XtSetSensitive(w, False);
-    if (!print_popup)
+    if (print_popup) {
+	/* the print popup already exists, but the magnification may have been
+	   changed in the export popup */
+	sprintf(buf,"%.2f",appres.magnification);
+	FirstArg(XtNstring, buf);
+	SetValues(print_mag_text);
+	/* also the figure size (magnification * bounding_box) */
+	print_update_figure_size();
+    } else {
 	create_print_panel(w);
+    }
     XtPopup(print_popup, XtGrabNone);
     /* insure that the most recent colormap is installed */
     set_cmap(XtWindow(print_popup));
@@ -293,10 +482,16 @@ create_print_panel(w)
     Widget	    w;
 {
 	Widget	    image;
+	Widget	    entry;
 	Pixmap	    p;
 	DeclareArgs(10);
 	unsigned    long fg, bg;
 	char	   *printer_val;
+	char	    buf[50];
+	char	   *unit;
+	int	    ux,uy,lx,ly;
+	int	    i;
+	float	    mult;
 
 	print_w = w;
 	XtTranslateCoords(w, (Position) 0, (Position) 0, &xposn, &yposn);
@@ -309,8 +504,8 @@ create_print_panel(w)
 					 transientShellWidgetClass,
 					 tool, Args, ArgCount);
         XtOverrideTranslations(print_popup,
-                           XtParseTranslationTable(prin_translations));
-        XtAppAddActions(tool_app, prin_actions, XtNumber(prin_actions));
+                           XtParseTranslationTable(prn_translations));
+        XtAppAddActions(tool_app, prn_actions, XtNumber(prn_actions));
 
 	print_panel = XtCreateManagedWidget("print_panel", formWidgetClass,
 					    print_popup, NULL, ZERO);
@@ -323,16 +518,14 @@ create_print_panel(w)
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNinternalWidth, 0);
 	NextArg(XtNinternalHeight, 0);
-	NextArg(XtNresize, False);
-	NextArg(XtNresizable, False);
 	image = XtCreateManagedWidget("printer_image", labelWidgetClass,
 				      print_panel, Args, ArgCount);
 	FirstArg(XtNforeground, &fg);
 	NextArg(XtNbackground, &bg);
 	GetValues(image);
 	p = XCreatePixmapFromBitmapData(tool_d, XtWindow(canvas_sw),
-		      (char *) printer_ic.data, printer_ic.width, printer_ic.height,
-		      fg, bg, DefaultDepthOfScreen(tool_s));
+		      printer_ic.bits, printer_ic.width, printer_ic.height,
+		      fg, bg, tool_dpth);
 	FirstArg(XtNbitmap, p);
 	SetValues(image);
 
@@ -343,34 +536,95 @@ create_print_panel(w)
 	(void) XtCreateManagedWidget("print_label", labelWidgetClass,
 					print_panel, Args, ArgCount);
 
-	FirstArg(XtNlabel, "    Magnification%:");
+	FirstArg(XtNlabel, " Magnification %");
 	NextArg(XtNfromVert, image);
 	NextArg(XtNjustify, XtJustifyLeft);
 	NextArg(XtNborderWidth, 0);
 	mag_lab = XtCreateManagedWidget("mag_label", labelWidgetClass,
 					print_panel, Args, ArgCount);
 
-	FirstArg(XtNwidth, 40);
+	FirstArg(XtNwidth, 60);
 	NextArg(XtNfromVert, image);
 	NextArg(XtNfromHoriz, mag_lab);
 	NextArg(XtNeditType, XawtextEdit);
-	NextArg(XtNstring, "100");
+	sprintf(buf, "%.2f", appres.magnification);
+	NextArg(XtNstring, buf);
 	NextArg(XtNinsertPosition, 3);
 	NextArg(XtNborderWidth, INTERNAL_BW);
-	mag_text = XtCreateManagedWidget("magnification", asciiTextWidgetClass,
+	/* we want to track typing here to update figure size label */
+	NextArg(XtNcallback, mag_callback);
+	print_mag_text = XtCreateManagedWidget("magnification", asciiTextWidgetClass,
 					 print_panel, Args, ArgCount);
-	XtOverrideTranslations(mag_text,
-			       XtParseTranslationTable(text_translations));
+	XtOverrideTranslations(print_mag_text,
+			       XtParseTranslationTable(print_translations));
 
-	FirstArg(XtNlabel, "       Orientation:");
+	/* Fit Page to the right of the magnification window */
+
+	FirstArg(XtNlabel, "Fit to Page");
+	NextArg(XtNfromVert, image);
+	NextArg(XtNfromHoriz, print_mag_text);
+	NextArg(XtNborderWidth, INTERNAL_BW);
+	fitpage = XtCreateManagedWidget("fitpage", commandWidgetClass,
+				       print_panel, Args, ArgCount);
+	XtAddEventHandler(fitpage, ButtonReleaseMask, (Boolean) 0,
+			  (XtEventHandler)fit_page, (XtPointer) NULL);
+
+	/* Figure Size to the right of the fit page window */
+
+	mult = appres.INCHES? PIX_PER_INCH : PIX_PER_CM;
+	unit = appres.INCHES? "in": "cm";
+	/* get the size of the figure */
+	compound_bound(&objects, &lx, &ly, &ux, &uy);
+	sprintf(buf, "Fig Size: %.1f%s x %.1f%s      ",
+		(float)(ux-lx)/mult*appres.magnification/100.0,unit,
+		(float)(uy-ly)/mult*appres.magnification/100.0,unit);
+	FirstArg(XtNlabel, buf);
+	NextArg(XtNfromVert, image);
+	NextArg(XtNfromHoriz, fitpage);
+	NextArg(XtNhorizDistance, 5);
 	NextArg(XtNjustify, XtJustifyLeft);
 	NextArg(XtNborderWidth, 0);
-	NextArg(XtNfromVert, mag_text);
+	size_lab = XtCreateManagedWidget("size_label", labelWidgetClass,
+					print_panel, Args, ArgCount);
+
+	/* paper size */
+
+	FirstArg(XtNlabel, "      Paper Size");
+	NextArg(XtNjustify, XtJustifyLeft);
+	NextArg(XtNborderWidth, 0);
+	NextArg(XtNfromVert, fitpage);
+	papersize_lab = XtCreateManagedWidget("papersize_label", labelWidgetClass,
+					 print_panel, Args, ArgCount);
+
+	FirstArg(XtNlabel, paper_sizes[appres.papersize].fname);
+	NextArg(XtNfromHoriz, papersize_lab);
+	NextArg(XtNfromVert, fitpage);
+	NextArg(XtNborderWidth, INTERNAL_BW);
+	NextArg(XtNresizable, True);
+	print_papersize_panel = XtCreateManagedWidget("papersize",
+					   menuButtonWidgetClass,
+					   print_panel, Args, ArgCount);
+	papersize_menu = XtCreatePopupShell("menu", simpleMenuWidgetClass, 
+				    print_papersize_panel, NULL, ZERO);
+
+	/* make the menu items */
+	for (i = 0; i < XtNumber(paper_sizes); i++) {
+	    entry = XtCreateManagedWidget(paper_sizes[i].fname, smeBSBObjectClass, 
+					papersize_menu, NULL, ZERO);
+	    XtAddCallback(entry, XtNcallback, papersize_select, (XtPointer) i);
+	}
+
+	/* Orientation */
+
+	FirstArg(XtNlabel, "     Orientation");
+	NextArg(XtNjustify, XtJustifyLeft);
+	NextArg(XtNborderWidth, 0);
+	NextArg(XtNfromVert, print_papersize_panel);
 	orient_lab = XtCreateManagedWidget("orient_label", labelWidgetClass,
 					   print_panel, Args, ArgCount);
 
 	FirstArg(XtNfromHoriz, orient_lab);
-	NextArg(XtNfromVert, mag_text);
+	NextArg(XtNfromVert, print_papersize_panel);
 	NextArg(XtNborderWidth, INTERNAL_BW);
 	print_orient_panel = XtCreateManagedWidget(orient_items[appres.landscape],
 					     menuButtonWidgetClass,
@@ -378,7 +632,7 @@ create_print_panel(w)
 	orient_menu = make_popup_menu(orient_items, XtNumber(orient_items),
 				      print_orient_panel, orient_select);
 
-	FirstArg(XtNlabel, "     Justification:");
+	FirstArg(XtNlabel, "   Justification");
 	NextArg(XtNjustify, XtJustifyLeft);
 	NextArg(XtNborderWidth, 0);
 	NextArg(XtNfromVert, print_orient_panel);
@@ -389,16 +643,35 @@ create_print_panel(w)
 	NextArg(XtNfromHoriz, just_lab);
 	NextArg(XtNfromVert, print_orient_panel);
 	NextArg(XtNborderWidth, INTERNAL_BW);
-	NextArg(XtNresizable, True);
 	print_just_panel = XtCreateManagedWidget("justify",
 					   menuButtonWidgetClass,
 					   print_panel, Args, ArgCount);
 	just_menu = make_popup_menu(just_items, XtNumber(just_items),
 				    print_just_panel, just_select);
 
+	/* multiple/single page */
 
-	FirstArg(XtNlabel, "PostScript Printer:");
+	FirstArg(XtNlabel, "           Pages");
+	NextArg(XtNjustify, XtJustifyLeft);
+	NextArg(XtNborderWidth, 0);
 	NextArg(XtNfromVert, print_just_panel);
+	multiple_lab = XtCreateManagedWidget("multiple_label", labelWidgetClass,
+					 print_panel, Args, ArgCount);
+
+	FirstArg(XtNlabel, multiple_pages[appres.multiple? 1:0]);
+	NextArg(XtNfromHoriz, multiple_lab);
+	NextArg(XtNfromVert, print_just_panel);
+	NextArg(XtNborderWidth, INTERNAL_BW);
+	print_multiple_panel = XtCreateManagedWidget("multiple_pages",
+					   menuButtonWidgetClass,
+					   print_panel, Args, ArgCount);
+	multiple_menu = make_popup_menu(multiple_pages, XtNumber(multiple_pages),
+				    print_multiple_panel, multiple_select);
+
+	/* printer name */
+
+	FirstArg(XtNlabel, "         Printer");
+	NextArg(XtNfromVert, print_multiple_panel);
 	NextArg(XtNjustify, XtJustifyLeft);
 	NextArg(XtNborderWidth, 0);
 	printer_lab = XtCreateManagedWidget("printer_label", labelWidgetClass,
@@ -409,7 +682,7 @@ create_print_panel(w)
 	 */
 
 	FirstArg(XtNwidth, 100);
-	NextArg(XtNfromVert, print_just_panel);
+	NextArg(XtNfromVert, print_multiple_panel);
 	NextArg(XtNfromHoriz, printer_lab);
 	NextArg(XtNeditType, XawtextEdit);
 	NextArg(XtNinsertPosition, 0);
@@ -418,7 +691,7 @@ create_print_panel(w)
 					     print_panel, Args, ArgCount);
 
 	XtOverrideTranslations(printer_text,
-			       XtParseTranslationTable(text_translations));
+			       XtParseTranslationTable(print_translations));
 
 	/* put the printer name in the label if resource isn't set */
 	FirstArg(XtNstring, &printer_val);
@@ -435,7 +708,7 @@ create_print_panel(w)
 		}
 	}
 
-	FirstArg(XtNlabel, "  Print Job Params:");
+	FirstArg(XtNlabel, "Print Job Params");
 	NextArg(XtNfromVert, printer_text);
 	NextArg(XtNjustify, XtJustifyLeft);
 	NextArg(XtNborderWidth, 0);
@@ -456,9 +729,9 @@ create_print_panel(w)
 					     print_panel, Args, ArgCount);
 
 	XtOverrideTranslations(param_text,
-			       XtParseTranslationTable(text_translations));
+			       XtParseTranslationTable(print_translations));
 
-	FirstArg(XtNlabel, "  Figures in batch:");
+	FirstArg(XtNlabel, "Figures in batch");
 	NextArg(XtNfromVert, param_text);
 	NextArg(XtNjustify, XtJustifyLeft);
 	NextArg(XtNborderWidth, 0);
@@ -477,7 +750,7 @@ create_print_panel(w)
 	NextArg(XtNfromVert, num_batch);
 	NextArg(XtNvertDistance, 10);
 	NextArg(XtNhorizDistance, 6);
-	NextArg(XtNheight, 30);
+	NextArg(XtNheight, 35);
 	NextArg(XtNborderWidth, INTERNAL_BW);
 	dismiss = XtCreateManagedWidget("dismiss", commandWidgetClass,
 				       print_panel, Args, ArgCount);
@@ -487,7 +760,8 @@ create_print_panel(w)
 	FirstArg(XtNlabel, "Print FIGURE\nto Printer");
 	NextArg(XtNfromVert, num_batch);
 	NextArg(XtNfromHoriz, dismiss);
-	NextArg(XtNheight, 30);
+	NextArg(XtNresize, False);	/* must not allow resize because the label changes */
+	NextArg(XtNheight, 35);
 	NextArg(XtNborderWidth, INTERNAL_BW);
 	NextArg(XtNvertDistance, 10);
 	NextArg(XtNhorizDistance, 6);
@@ -499,7 +773,7 @@ create_print_panel(w)
 	FirstArg(XtNlabel, "Print FIGURE\nto Batch");
 	NextArg(XtNfromVert, num_batch);
 	NextArg(XtNfromHoriz, print);
-	NextArg(XtNheight, 30);
+	NextArg(XtNheight, 35);
 	NextArg(XtNborderWidth, INTERNAL_BW);
 	NextArg(XtNvertDistance, 10);
 	NextArg(XtNhorizDistance, 6);
@@ -511,7 +785,7 @@ create_print_panel(w)
 	FirstArg(XtNlabel, "Clear\nBatch");
 	NextArg(XtNfromVert, num_batch);
 	NextArg(XtNfromHoriz, print_batch);
-	NextArg(XtNheight, 30);
+	NextArg(XtNheight, 35);
 	NextArg(XtNborderWidth, INTERNAL_BW);
 	NextArg(XtNvertDistance, 10);
 	NextArg(XtNhorizDistance, 6);
@@ -525,4 +799,21 @@ create_print_panel(w)
 	XtInstallAccelerators(print_panel, clear_batch);
 	XtInstallAccelerators(print_panel, print);
 	update_batch_count();
+
+	/* if multiple pages is on, desensitive justification panels */
+	if (appres.multiple) {
+	    XtSetSensitive(just_lab, False);
+	    XtSetSensitive(print_just_panel, False);
+	    if (export_just_panel) {
+	        XtSetSensitive(just_lab, False);
+	        XtSetSensitive(export_just_panel, False);
+	    }
+	} else {
+	    XtSetSensitive(just_lab, True);
+	    XtSetSensitive(print_just_panel, True);
+	    if (export_just_panel) {
+	        XtSetSensitive(just_lab, True);
+	        XtSetSensitive(export_just_panel, True);
+	    }
+	}
 }
