@@ -10,11 +10,14 @@
  * nonexclusive right and license to deal in this software and
  * documentation files (the "Software"), including without limitation the
  * rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons who receive
- * copies from any such party to do so, with the only requirement being
- * that this copyright notice remain intact.  This license includes without
- * limitation a license to do the foregoing actions under any patents of
- * the party supplying this software to the X Consortium.
+ * and/or sell copies of the Software subject to the restriction stated
+ * below, and to permit persons who receive copies from any such party to
+ * do so, with the only requirement being that this copyright notice remain
+ * intact.
+ * This license includes without limitation a license to do the foregoing
+ * actions under any patents of the party supplying this software to the 
+ * X Consortium.
+ *
  */
 
 #include "fig.h"
@@ -33,6 +36,8 @@ int		MSGPANEL_WD = 600;
 int		MOUSEFUN_WD, MOUSEFUN_HT;
 int		INDPANEL_WD;
 int		CANVAS_WD, CANVAS_HT;
+int		CANVAS_WD_LAND, CANVAS_HT_LAND;
+int		CANVAS_WD_PORT, CANVAS_HT_PORT;
 int		INTERNAL_BW;
 int		TOPRULER_WD, TOPRULER_HT;
 int		SIDERULER_WD, SIDERULER_HT;
